""" Contains all the data models used in inputs/outputs """

from .batch import Batch
from .color_name import ColorName
from .comment import Comment
from .comment_create import CommentCreate
from .comment_create_text import CommentCreateText
from .comment_reaction import CommentReaction
from .comment_reaction_create import CommentReactionCreate
from .comment_reaction_update import CommentReactionUpdate
from .comment_text import CommentText
from .comment_update import CommentUpdate
from .comment_update_text import CommentUpdateText
from .cycle_mode import CycleMode
from .dartboard import Dartboard
from .dartboard_create import DartboardCreate
from .dartboard_kind import DartboardKind
from .dartboard_update import DartboardUpdate
from .discord_integration import DiscordIntegration
from .doc import Doc
from .doc_create import DocCreate
from .doc_create_text import DocCreateText
from .doc_source_type import DocSourceType
from .doc_text import DocText
from .doc_update import DocUpdate
from .doc_update_text import DocUpdateText
from .event import Event
from .event_adtl import EventAdtl
from .event_create import EventCreate
from .event_create_adtl import EventCreateAdtl
from .event_kind import EventKind
from .event_subscription import EventSubscription
from .event_subscription_create import EventSubscriptionCreate
from .event_subscription_update import EventSubscriptionUpdate
from .event_update import EventUpdate
from .event_update_adtl import EventUpdateAdtl
from .filter_applicability import FilterApplicability
from .filter_assignee import FilterAssignee
from .filter_connector import FilterConnector
from .filter_group import FilterGroup
from .filter_search import FilterSearch
from .filter_set import FilterSet
from .folder import Folder
from .folder_create import FolderCreate
from .folder_kind import FolderKind
from .folder_update import FolderUpdate
from .github_integration import GithubIntegration
from .github_integration_tenant_extension_status import GithubIntegrationTenantExtensionStatus
from .google_data import GoogleData
from .icon_kind import IconKind
from .layout import Layout
from .layout_config import LayoutConfig
from .layout_create import LayoutCreate
from .layout_create_filter_group import LayoutCreateFilterGroup
from .layout_create_kind_config_map import LayoutCreateKindConfigMap
from .layout_create_sorts import LayoutCreateSorts
from .layout_kind import LayoutKind
from .layout_kind_config_map import LayoutKindConfigMap
from .layout_update import LayoutUpdate
from .layout_update_filter_group import LayoutUpdateFilterGroup
from .layout_update_kind_config_map import LayoutUpdateKindConfigMap
from .layout_update_sorts import LayoutUpdateSorts
from .models_response import ModelsResponse
from .notion_integration import NotionIntegration
from .notion_integration_tenant_extension_status import NotionIntegrationTenantExtensionStatus
from .operation import Operation
from .operation_kind import OperationKind
from .operation_model_kind import OperationModelKind
from .priority import Priority
from .recommendation_status import RecommendationStatus
from .relationship import Relationship
from .relationship_create import RelationshipCreate
from .relationship_kind import RelationshipKind
from .relationship_kind_create import RelationshipKindCreate
from .relationship_kind_kind import RelationshipKindKind
from .relationship_kind_update import RelationshipKindUpdate
from .relationship_update import RelationshipUpdate
from .request_body import RequestBody
from .response_body import ResponseBody
from .slack_integration import SlackIntegration
from .slack_integration_tenant_extension_status import SlackIntegrationTenantExtensionStatus
from .sort import Sort
from .space import Space
from .space_create import SpaceCreate
from .space_kind import SpaceKind
from .space_update import SpaceUpdate
from .status import Status
from .status_create import StatusCreate
from .status_kind import StatusKind
from .status_update import StatusUpdate
from .subscription import Subscription
from .subtask_display_mode import SubtaskDisplayMode
from .summary_statistic_kind import SummaryStatisticKind
from .tag import Tag
from .tag_create import TagCreate
from .tag_update import TagUpdate
from .task import Task
from .task_attachment import TaskAttachment
from .task_attachment_create import TaskAttachmentCreate
from .task_attachment_update import TaskAttachmentUpdate
from .task_create import TaskCreate
from .task_create_description import TaskCreateDescription
from .task_description import TaskDescription
from .task_doc_relationship import TaskDocRelationship
from .task_doc_relationship_create import TaskDocRelationshipCreate
from .task_doc_relationship_update import TaskDocRelationshipUpdate
from .task_link import TaskLink
from .task_link_adtl import TaskLinkAdtl
from .task_link_create import TaskLinkCreate
from .task_link_kind import TaskLinkKind
from .task_link_update import TaskLinkUpdate
from .task_notion_document import TaskNotionDocument
from .task_notion_document_block_children_map import TaskNotionDocumentBlockChildrenMap
from .task_notion_document_block_map import TaskNotionDocumentBlockMap
from .task_notion_document_page_map import TaskNotionDocumentPageMap
from .task_source_type import TaskSourceType
from .task_update import TaskUpdate
from .task_update_description import TaskUpdateDescription
from .tenant import Tenant
from .tenant_create import TenantCreate
from .tenant_entitlement_overrides import TenantEntitlementOverrides
from .tenant_update import TenantUpdate
from .theme import Theme
from .transaction import Transaction
from .transaction_kind import TransactionKind
from .transaction_response import TransactionResponse
from .user import User
from .user_create import UserCreate
from .user_dartboard_layout import UserDartboardLayout
from .user_dartboard_layout_create import UserDartboardLayoutCreate
from .user_dartboard_layout_update import UserDartboardLayoutUpdate
from .user_status import UserStatus
from .user_update import UserUpdate
from .validation_error_response import ValidationErrorResponse
from .validation_error_response_items import ValidationErrorResponseItems
from .view import View
from .view_create import ViewCreate
from .view_kind import ViewKind
from .view_update import ViewUpdate
from .yc_integration import YcIntegration

__all__ = (
    "Batch",
    "ColorName",
    "Comment",
    "CommentCreate",
    "CommentCreateText",
    "CommentReaction",
    "CommentReactionCreate",
    "CommentReactionUpdate",
    "CommentText",
    "CommentUpdate",
    "CommentUpdateText",
    "CycleMode",
    "Dartboard",
    "DartboardCreate",
    "DartboardKind",
    "DartboardUpdate",
    "DiscordIntegration",
    "Doc",
    "DocCreate",
    "DocCreateText",
    "DocSourceType",
    "DocText",
    "DocUpdate",
    "DocUpdateText",
    "Event",
    "EventAdtl",
    "EventCreate",
    "EventCreateAdtl",
    "EventKind",
    "EventSubscription",
    "EventSubscriptionCreate",
    "EventSubscriptionUpdate",
    "EventUpdate",
    "EventUpdateAdtl",
    "FilterApplicability",
    "FilterAssignee",
    "FilterConnector",
    "FilterGroup",
    "FilterSearch",
    "FilterSet",
    "Folder",
    "FolderCreate",
    "FolderKind",
    "FolderUpdate",
    "GithubIntegration",
    "GithubIntegrationTenantExtensionStatus",
    "GoogleData",
    "IconKind",
    "Layout",
    "LayoutConfig",
    "LayoutCreate",
    "LayoutCreateFilterGroup",
    "LayoutCreateKindConfigMap",
    "LayoutCreateSorts",
    "LayoutKind",
    "LayoutKindConfigMap",
    "LayoutUpdate",
    "LayoutUpdateFilterGroup",
    "LayoutUpdateKindConfigMap",
    "LayoutUpdateSorts",
    "ModelsResponse",
    "NotionIntegration",
    "NotionIntegrationTenantExtensionStatus",
    "Operation",
    "OperationKind",
    "OperationModelKind",
    "Priority",
    "RecommendationStatus",
    "Relationship",
    "RelationshipCreate",
    "RelationshipKind",
    "RelationshipKindCreate",
    "RelationshipKindKind",
    "RelationshipKindUpdate",
    "RelationshipUpdate",
    "RequestBody",
    "ResponseBody",
    "SlackIntegration",
    "SlackIntegrationTenantExtensionStatus",
    "Sort",
    "Space",
    "SpaceCreate",
    "SpaceKind",
    "SpaceUpdate",
    "Status",
    "StatusCreate",
    "StatusKind",
    "StatusUpdate",
    "Subscription",
    "SubtaskDisplayMode",
    "SummaryStatisticKind",
    "Tag",
    "TagCreate",
    "TagUpdate",
    "Task",
    "TaskAttachment",
    "TaskAttachmentCreate",
    "TaskAttachmentUpdate",
    "TaskCreate",
    "TaskCreateDescription",
    "TaskDescription",
    "TaskDocRelationship",
    "TaskDocRelationshipCreate",
    "TaskDocRelationshipUpdate",
    "TaskLink",
    "TaskLinkAdtl",
    "TaskLinkCreate",
    "TaskLinkKind",
    "TaskLinkUpdate",
    "TaskNotionDocument",
    "TaskNotionDocumentBlockChildrenMap",
    "TaskNotionDocumentBlockMap",
    "TaskNotionDocumentPageMap",
    "TaskSourceType",
    "TaskUpdate",
    "TaskUpdateDescription",
    "Tenant",
    "TenantCreate",
    "TenantEntitlementOverrides",
    "TenantUpdate",
    "Theme",
    "Transaction",
    "TransactionKind",
    "TransactionResponse",
    "User",
    "UserCreate",
    "UserDartboardLayout",
    "UserDartboardLayoutCreate",
    "UserDartboardLayoutUpdate",
    "UserStatus",
    "UserUpdate",
    "ValidationErrorResponse",
    "ValidationErrorResponseItems",
    "View",
    "ViewCreate",
    "ViewKind",
    "ViewUpdate",
    "YcIntegration",
)



from enum import Enum


class UserStatus(str, Enum):
    ACTIVE = "Active"
    DEACTIVATED = "Deactivated"
    INVITED = "Invited"
    PENDINGEMAILVERIFICATION = "PendingEmailVerification"
    PENDINGSUBSCRIPTIONUPGRADE = "PendingSubscriptionUpgrade"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Optional, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.task_link_kind import TaskLinkKind

if TYPE_CHECKING:
    from ..models.task_link_adtl import TaskLinkAdtl


T = TypeVar("T", bound="TaskLink")


@_attrs_define
class TaskLink:
    """
    Attributes:
        duid (str):
        order (str):
        kind (TaskLinkKind): * `Standard` - STANDARD
            * `Source From Template` - SOURCE_FROM_TEMPLATE
            * `Source From Third Party` - SOURCE_FROM_THIRD_PARTY
            * `GitHub Branch` - GITHUB_BRANCH
            * `GitHub Pull Request` - GITHUB_PULL_REQUEST
            * `GitHub Expansion` - GITHUB_EXPANSION
            * `Notion Link` - NOTION_LINK
            * `Notion Document` - NOTION_DOCUMENT
            * `Notion Document Doesnt Exist` - NOTION_DOCUMENT_DOESNT_EXIST
            * `Notion Document Parse Failed` - NOTION_DOCUMENT_PARSE_FAILED
            * `Notion Expansion` - NOTION_EXPANSION
            * `Slack Expansion` - SLACK_EXPANSION
        url (str):
        adtl (TaskLinkAdtl):
        title (Optional[str]):
        icon_url (Optional[str]):
    """

    duid: str
    order: str
    kind: TaskLinkKind
    url: str
    adtl: "TaskLinkAdtl"
    title: Optional[str]
    icon_url: Optional[str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid
        order = self.order
        kind = self.kind.value

        url = self.url
        adtl = self.adtl.to_dict()

        title = self.title
        icon_url = self.icon_url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "order": order,
                "kind": kind,
                "url": url,
                "adtl": adtl,
                "title": title,
                "iconUrl": icon_url,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.task_link_adtl import TaskLinkAdtl

        d = src_dict.copy()
        duid = d.pop("duid")

        order = d.pop("order")

        kind = TaskLinkKind(d.pop("kind"))

        url = d.pop("url")

        adtl = TaskLinkAdtl.from_dict(d.pop("adtl"))

        title = d.pop("title")

        icon_url = d.pop("iconUrl")

        task_link = cls(
            duid=duid,
            order=order,
            kind=kind,
            url=url,
            adtl=adtl,
            title=title,
            icon_url=icon_url,
        )

        task_link.additional_properties = d
        return task_link

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.comment_create_text import CommentCreateText


T = TypeVar("T", bound="CommentCreate")


@_attrs_define
class CommentCreate:
    """
    Attributes:
        duid (str):
        author_duid (str):
        task_duid (str):
        root_duid (Union[Unset, None, str]):
        text (Union[Unset, CommentCreateText]):
        published_at (Union[Unset, None, datetime.datetime]):
    """

    duid: str
    author_duid: str
    task_duid: str
    root_duid: Union[Unset, None, str] = UNSET
    text: Union[Unset, "CommentCreateText"] = UNSET
    published_at: Union[Unset, None, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid
        author_duid = self.author_duid
        task_duid = self.task_duid
        root_duid = self.root_duid
        text: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.text, Unset):
            text = self.text.to_dict()

        published_at: Union[Unset, None, str] = UNSET
        if not isinstance(self.published_at, Unset):
            published_at = self.published_at.isoformat() if self.published_at else None

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "authorDuid": author_duid,
                "taskDuid": task_duid,
            }
        )
        if root_duid is not UNSET:
            field_dict["rootDuid"] = root_duid
        if text is not UNSET:
            field_dict["text"] = text
        if published_at is not UNSET:
            field_dict["publishedAt"] = published_at

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.comment_create_text import CommentCreateText

        d = src_dict.copy()
        duid = d.pop("duid")

        author_duid = d.pop("authorDuid")

        task_duid = d.pop("taskDuid")

        root_duid = d.pop("rootDuid", UNSET)

        _text = d.pop("text", UNSET)
        text: Union[Unset, CommentCreateText]
        if isinstance(_text, Unset):
            text = UNSET
        else:
            text = CommentCreateText.from_dict(_text)

        _published_at = d.pop("publishedAt", UNSET)
        published_at: Union[Unset, None, datetime.datetime]
        if _published_at is None:
            published_at = None
        elif isinstance(_published_at, Unset):
            published_at = UNSET
        else:
            published_at = isoparse(_published_at)

        comment_create = cls(
            duid=duid,
            author_duid=author_duid,
            task_duid=task_duid,
            root_duid=root_duid,
            text=text,
            published_at=published_at,
        )

        comment_create.additional_properties = d
        return comment_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="UserDartboardLayoutUpdate")


@_attrs_define
class UserDartboardLayoutUpdate:
    """
    Attributes:
        user_duid (Union[Unset, str]):
        dartboard_duid (Union[Unset, str]):
        layout_duid (Union[Unset, str]):
    """

    user_duid: Union[Unset, str] = UNSET
    dartboard_duid: Union[Unset, str] = UNSET
    layout_duid: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        user_duid = self.user_duid
        dartboard_duid = self.dartboard_duid
        layout_duid = self.layout_duid

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if user_duid is not UNSET:
            field_dict["userDuid"] = user_duid
        if dartboard_duid is not UNSET:
            field_dict["dartboardDuid"] = dartboard_duid
        if layout_duid is not UNSET:
            field_dict["layoutDuid"] = layout_duid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        user_duid = d.pop("userDuid", UNSET)

        dartboard_duid = d.pop("dartboardDuid", UNSET)

        layout_duid = d.pop("layoutDuid", UNSET)

        user_dartboard_layout_update = cls(
            user_duid=user_duid,
            dartboard_duid=dartboard_duid,
            layout_duid=layout_duid,
        )

        user_dartboard_layout_update.additional_properties = d
        return user_dartboard_layout_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DocSourceType(str, Enum):
    APPLICATION = "Application"
    RECOMMENDATION = "Recommendation"
    UNKNOWN = "Unknown"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class OperationKind(str, Enum):
    CREATE = "create"
    DELETE = "delete"
    UPDATE = "update"
    UPDATE_LIST_ADD = "update_list_add"
    UPDATE_LIST_REMOVE = "update_list_remove"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.layout_kind import LayoutKind
from ..models.summary_statistic_kind import SummaryStatisticKind
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.filter_group import FilterGroup
    from ..models.layout_kind_config_map import LayoutKindConfigMap
    from ..models.sort import Sort


T = TypeVar("T", bound="Layout")


@_attrs_define
class Layout:
    """
    Attributes:
        duid (str):
        kind (LayoutKind): * `list` - LIST
            * `board` - BOARD
            * `calendar` - CALENDAR
        kind_config_map (LayoutKindConfigMap):
        filter_group (FilterGroup):
        sorts (List['Sort']):
        summary_statistic_kind (SummaryStatisticKind): * `None` - NONE
            * `TotalCount` - TOTAL_COUNT
            * `IncompleteCount` - INCOMPLETE_COUNT
            * `CompletedPercent` - COMPLETED_PERCENT
            * `TotalCountPoints` - TOTAL_COUNT_POINTS
            * `IncompleteCountPoints` - INCOMPLETE_COUNT_POINTS
            * `CompletedPercentPoints` - COMPLETED_PERCENT_POINTS
        updated_by_client_duid (Union[Unset, None, str]):
    """

    duid: str
    kind: LayoutKind
    kind_config_map: "LayoutKindConfigMap"
    filter_group: "FilterGroup"
    sorts: List["Sort"]
    summary_statistic_kind: SummaryStatisticKind
    updated_by_client_duid: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid
        kind = self.kind.value

        kind_config_map = self.kind_config_map.to_dict()

        filter_group = self.filter_group.to_dict()

        sorts = []
        for sorts_item_data in self.sorts:
            sorts_item = sorts_item_data.to_dict()

            sorts.append(sorts_item)

        summary_statistic_kind = self.summary_statistic_kind.value

        updated_by_client_duid = self.updated_by_client_duid

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "kind": kind,
                "kindConfigMap": kind_config_map,
                "filterGroup": filter_group,
                "sorts": sorts,
                "summaryStatisticKind": summary_statistic_kind,
            }
        )
        if updated_by_client_duid is not UNSET:
            field_dict["updatedByClientDuid"] = updated_by_client_duid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.filter_group import FilterGroup
        from ..models.layout_kind_config_map import LayoutKindConfigMap
        from ..models.sort import Sort

        d = src_dict.copy()
        duid = d.pop("duid")

        kind = LayoutKind(d.pop("kind"))

        kind_config_map = LayoutKindConfigMap.from_dict(d.pop("kindConfigMap"))

        filter_group = FilterGroup.from_dict(d.pop("filterGroup"))

        sorts = []
        _sorts = d.pop("sorts")
        for sorts_item_data in _sorts:
            sorts_item = Sort.from_dict(sorts_item_data)

            sorts.append(sorts_item)

        summary_statistic_kind = SummaryStatisticKind(d.pop("summaryStatisticKind"))

        updated_by_client_duid = d.pop("updatedByClientDuid", UNSET)

        layout = cls(
            duid=duid,
            kind=kind,
            kind_config_map=kind_config_map,
            filter_group=filter_group,
            sorts=sorts,
            summary_statistic_kind=summary_statistic_kind,
            updated_by_client_duid=updated_by_client_duid,
        )

        layout.additional_properties = d
        return layout

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Optional, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.color_name import ColorName
from ..models.theme import Theme
from ..models.user_status import UserStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_data import GoogleData


T = TypeVar("T", bound="User")


@_attrs_define
class User:
    """
    Attributes:
        duid (str):
        status (UserStatus): * `PendingSubscriptionUpgrade` - PENDING_SUBSCRIPTION_UPGRADE
            * `Invited` - INVITED
            * `PendingEmailVerification` - PENDING_EMAIL_VERIFICATION
            * `Active` - ACTIVE
            * `Deactivated` - DEACTIVATED
        email (str):
        name (str):
        abrev (str):
        theme (Theme): * `System Default` - SYSTEM_DEFAULT
            * `Light` - LIGHT
            * `Dark` - DARK
        color_name (ColorName): * `Red` - RED
            * `Dark Blue` - DARK_BLUE
            * `Dark Orange` - DARK_ORANGE
            * `Dark Green` - DARK_GREEN
            * `Purple` - PURPLE
            * `Dark Teal` - DARK_TEAL
            * `Pink` - PINK
            * `Orange` - ORANGE
            * `Green` - GREEN
            * `Yellow` - YELLOW
            * `Brown` - BROWN
            * `Dark Red` - DARK_RED
            * `Flat Green` - FLAT_GREEN
            * `Red Orange` - RED_ORANGE
            * `Teal` - TEAL
            * `Light Green` - LIGHT_GREEN
            * `Light Blue` - LIGHT_BLUE
            * `Light Purple` - LIGHT_PURPLE
            * `Light Orange` - LIGHT_ORANGE
            * `Light Pink` - LIGHT_PINK
            * `Tan` - TAN
            * `Dark Gray` - DARK_GRAY
            * `Light Brown` - LIGHT_BROWN
            * `Light Gray` - LIGHT_GRAY
        is_admin (bool):
        updated_by_client_duid (Union[Unset, None, str]):
        image_url (Optional[str]):
        google_data (Optional[GoogleData]):
    """

    duid: str
    status: UserStatus
    email: str
    name: str
    abrev: str
    theme: Theme
    color_name: ColorName
    is_admin: bool
    image_url: Optional[str]
    google_data: Optional["GoogleData"]
    updated_by_client_duid: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid
        status = self.status.value

        email = self.email
        name = self.name
        abrev = self.abrev
        theme = self.theme.value

        color_name = self.color_name.value

        is_admin = self.is_admin
        updated_by_client_duid = self.updated_by_client_duid
        image_url = self.image_url
        google_data = self.google_data.to_dict() if self.google_data else None

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "status": status,
                "email": email,
                "name": name,
                "abrev": abrev,
                "theme": theme,
                "colorName": color_name,
                "isAdmin": is_admin,
                "imageUrl": image_url,
                "googleData": google_data,
            }
        )
        if updated_by_client_duid is not UNSET:
            field_dict["updatedByClientDuid"] = updated_by_client_duid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_data import GoogleData

        d = src_dict.copy()
        duid = d.pop("duid")

        status = UserStatus(d.pop("status"))

        email = d.pop("email")

        name = d.pop("name")

        abrev = d.pop("abrev")

        theme = Theme(d.pop("theme"))

        color_name = ColorName(d.pop("colorName"))

        is_admin = d.pop("isAdmin")

        updated_by_client_duid = d.pop("updatedByClientDuid", UNSET)

        image_url = d.pop("imageUrl")

        _google_data = d.pop("googleData")
        google_data: Optional[GoogleData]
        if _google_data is None:
            google_data = None
        else:
            google_data = GoogleData.from_dict(_google_data)

        user = cls(
            duid=duid,
            status=status,
            email=email,
            name=name,
            abrev=abrev,
            theme=theme,
            color_name=color_name,
            is_admin=is_admin,
            updated_by_client_duid=updated_by_client_duid,
            image_url=image_url,
            google_data=google_data,
        )

        user.additional_properties = d
        return user

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

T = TypeVar("T", bound="CommentReaction")


@_attrs_define
class CommentReaction:
    """
    Attributes:
        duid (str):
        created_at (datetime.datetime):
        author_duid (str):
        emoji (str):
    """

    duid: str
    created_at: datetime.datetime
    author_duid: str
    emoji: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid
        created_at = self.created_at.isoformat()

        author_duid = self.author_duid
        emoji = self.emoji

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "createdAt": created_at,
                "authorDuid": author_duid,
                "emoji": emoji,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        duid = d.pop("duid")

        created_at = isoparse(d.pop("createdAt"))

        author_duid = d.pop("authorDuid")

        emoji = d.pop("emoji")

        comment_reaction = cls(
            duid=duid,
            created_at=created_at,
            author_duid=author_duid,
            emoji=emoji,
        )

        comment_reaction.additional_properties = d
        return comment_reaction

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.layout_kind import LayoutKind
from ..models.summary_statistic_kind import SummaryStatisticKind
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.layout_create_filter_group import LayoutCreateFilterGroup
    from ..models.layout_create_kind_config_map import LayoutCreateKindConfigMap
    from ..models.layout_create_sorts import LayoutCreateSorts


T = TypeVar("T", bound="LayoutCreate")


@_attrs_define
class LayoutCreate:
    """
    Attributes:
        duid (str):
        kind (Union[Unset, LayoutKind]): * `list` - LIST
            * `board` - BOARD
            * `calendar` - CALENDAR
        kind_config_map (Union[Unset, LayoutCreateKindConfigMap]):
        filter_group (Union[Unset, LayoutCreateFilterGroup]):
        sorts (Union[Unset, LayoutCreateSorts]):
        summary_statistic_kind (Union[Unset, SummaryStatisticKind]): * `None` - NONE
            * `TotalCount` - TOTAL_COUNT
            * `IncompleteCount` - INCOMPLETE_COUNT
            * `CompletedPercent` - COMPLETED_PERCENT
            * `TotalCountPoints` - TOTAL_COUNT_POINTS
            * `IncompleteCountPoints` - INCOMPLETE_COUNT_POINTS
            * `CompletedPercentPoints` - COMPLETED_PERCENT_POINTS
    """

    duid: str
    kind: Union[Unset, LayoutKind] = UNSET
    kind_config_map: Union[Unset, "LayoutCreateKindConfigMap"] = UNSET
    filter_group: Union[Unset, "LayoutCreateFilterGroup"] = UNSET
    sorts: Union[Unset, "LayoutCreateSorts"] = UNSET
    summary_statistic_kind: Union[Unset, SummaryStatisticKind] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid
        kind: Union[Unset, str] = UNSET
        if not isinstance(self.kind, Unset):
            kind = self.kind.value

        kind_config_map: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.kind_config_map, Unset):
            kind_config_map = self.kind_config_map.to_dict()

        filter_group: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.filter_group, Unset):
            filter_group = self.filter_group.to_dict()

        sorts: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.sorts, Unset):
            sorts = self.sorts.to_dict()

        summary_statistic_kind: Union[Unset, str] = UNSET
        if not isinstance(self.summary_statistic_kind, Unset):
            summary_statistic_kind = self.summary_statistic_kind.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
            }
        )
        if kind is not UNSET:
            field_dict["kind"] = kind
        if kind_config_map is not UNSET:
            field_dict["kindConfigMap"] = kind_config_map
        if filter_group is not UNSET:
            field_dict["filterGroup"] = filter_group
        if sorts is not UNSET:
            field_dict["sorts"] = sorts
        if summary_statistic_kind is not UNSET:
            field_dict["summaryStatisticKind"] = summary_statistic_kind

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.layout_create_filter_group import LayoutCreateFilterGroup
        from ..models.layout_create_kind_config_map import LayoutCreateKindConfigMap
        from ..models.layout_create_sorts import LayoutCreateSorts

        d = src_dict.copy()
        duid = d.pop("duid")

        _kind = d.pop("kind", UNSET)
        kind: Union[Unset, LayoutKind]
        if isinstance(_kind, Unset):
            kind = UNSET
        else:
            kind = LayoutKind(_kind)

        _kind_config_map = d.pop("kindConfigMap", UNSET)
        kind_config_map: Union[Unset, LayoutCreateKindConfigMap]
        if isinstance(_kind_config_map, Unset):
            kind_config_map = UNSET
        else:
            kind_config_map = LayoutCreateKindConfigMap.from_dict(_kind_config_map)

        _filter_group = d.pop("filterGroup", UNSET)
        filter_group: Union[Unset, LayoutCreateFilterGroup]
        if isinstance(_filter_group, Unset):
            filter_group = UNSET
        else:
            filter_group = LayoutCreateFilterGroup.from_dict(_filter_group)

        _sorts = d.pop("sorts", UNSET)
        sorts: Union[Unset, LayoutCreateSorts]
        if isinstance(_sorts, Unset):
            sorts = UNSET
        else:
            sorts = LayoutCreateSorts.from_dict(_sorts)

        _summary_statistic_kind = d.pop("summaryStatisticKind", UNSET)
        summary_statistic_kind: Union[Unset, SummaryStatisticKind]
        if isinstance(_summary_statistic_kind, Unset):
            summary_statistic_kind = UNSET
        else:
            summary_statistic_kind = SummaryStatisticKind(_summary_statistic_kind)

        layout_create = cls(
            duid=duid,
            kind=kind,
            kind_config_map=kind_config_map,
            filter_group=filter_group,
            sorts=sorts,
            summary_statistic_kind=summary_statistic_kind,
        )

        layout_create.additional_properties = d
        return layout_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Optional, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="DiscordIntegration")


@_attrs_define
class DiscordIntegration:
    """
    Attributes:
        enabled (bool):
        webhook_url (Optional[str]):
    """

    enabled: bool
    webhook_url: Optional[str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        enabled = self.enabled
        webhook_url = self.webhook_url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "enabled": enabled,
                "webhookUrl": webhook_url,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        enabled = d.pop("enabled")

        webhook_url = d.pop("webhookUrl")

        discord_integration = cls(
            enabled=enabled,
            webhook_url=webhook_url,
        )

        discord_integration.additional_properties = d
        return discord_integration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class SlackIntegrationTenantExtensionStatus(str, Enum):
    DISABLED = "Disabled"
    ENABLED = "Enabled"
    PENDING = "Pending"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Optional, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.comment_reaction import CommentReaction
    from ..models.comment_text import CommentText


T = TypeVar("T", bound="Comment")


@_attrs_define
class Comment:
    """
    Attributes:
        duid (str):
        updated_at (datetime.datetime):
        task_duid (str):
        author_duid (str):
        text (CommentText):
        reactions (List['CommentReaction']):
        is_draft (bool):
        edited (bool):
        updated_by_client_duid (Union[Unset, None, str]):
        root_duid (Optional[str]):
        published_at (Optional[datetime.datetime]):
    """

    duid: str
    updated_at: datetime.datetime
    task_duid: str
    author_duid: str
    text: "CommentText"
    reactions: List["CommentReaction"]
    is_draft: bool
    edited: bool
    root_duid: Optional[str]
    published_at: Optional[datetime.datetime]
    updated_by_client_duid: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid
        updated_at = self.updated_at.isoformat()

        task_duid = self.task_duid
        author_duid = self.author_duid
        text = self.text.to_dict()

        reactions = []
        for reactions_item_data in self.reactions:
            reactions_item = reactions_item_data.to_dict()

            reactions.append(reactions_item)

        is_draft = self.is_draft
        edited = self.edited
        updated_by_client_duid = self.updated_by_client_duid
        root_duid = self.root_duid
        published_at = self.published_at.isoformat() if self.published_at else None

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "updatedAt": updated_at,
                "taskDuid": task_duid,
                "authorDuid": author_duid,
                "text": text,
                "reactions": reactions,
                "isDraft": is_draft,
                "edited": edited,
                "rootDuid": root_duid,
                "publishedAt": published_at,
            }
        )
        if updated_by_client_duid is not UNSET:
            field_dict["updatedByClientDuid"] = updated_by_client_duid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.comment_reaction import CommentReaction
        from ..models.comment_text import CommentText

        d = src_dict.copy()
        duid = d.pop("duid")

        updated_at = isoparse(d.pop("updatedAt"))

        task_duid = d.pop("taskDuid")

        author_duid = d.pop("authorDuid")

        text = CommentText.from_dict(d.pop("text"))

        reactions = []
        _reactions = d.pop("reactions")
        for reactions_item_data in _reactions:
            reactions_item = CommentReaction.from_dict(reactions_item_data)

            reactions.append(reactions_item)

        is_draft = d.pop("isDraft")

        edited = d.pop("edited")

        updated_by_client_duid = d.pop("updatedByClientDuid", UNSET)

        root_duid = d.pop("rootDuid")

        _published_at = d.pop("publishedAt")
        published_at: Optional[datetime.datetime]
        if _published_at is None:
            published_at = None
        else:
            published_at = isoparse(_published_at)

        comment = cls(
            duid=duid,
            updated_at=updated_at,
            task_duid=task_duid,
            author_duid=author_duid,
            text=text,
            reactions=reactions,
            is_draft=is_draft,
            edited=edited,
            updated_by_client_duid=updated_by_client_duid,
            root_duid=root_duid,
            published_at=published_at,
        )

        comment.additional_properties = d
        return comment

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.priority import Priority
from ..models.recommendation_status import RecommendationStatus
from ..models.task_source_type import TaskSourceType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.task_create_description import TaskCreateDescription


T = TypeVar("T", bound="TaskCreate")


@_attrs_define
class TaskCreate:
    """
    Attributes:
        duid (str):
        dartboard_duid (str):
        order (str):
        status_duid (str):
        source_user (Union[Unset, None, str]):
        source_type (Union[Unset, TaskSourceType]): * `Unknown` - UNKNOWN
            * `Import` - IMPORT
            * `Onboarding` - ONBOARDING
            * `Recommendation` - RECOMMENDATION
            * `Template` - TEMPLATE
            * `ChatGPT` - CHAT_GPT
            * `Email` - EMAIL
            * `Slack` - SLACK
            * `API` - API
            * `CLI` - CLI
            * `Application` - APPLICATION
            * `AppTcm` - APP_TCM
            * `AppQuickAdd` - APP_QUICK_ADD
            * `AppBoard` - APP_BOARD
            * `AppSubtask` - APP_SUBTASK
            * `AppRelationship` - APP_RELATIONSHIP
            * `AppEnter` - APP_ENTER
            * `AppReplicate` - APP_REPLICATE
            * `AppPaste` - APP_PASTE Default: TaskSourceType.UNKNOWN.
        drafter_duid (Union[Unset, None, str]):
        in_trash (Union[Unset, bool]):
        recommendation_status (Union[Unset, None, RecommendationStatus]): * `Accepted` - ACCEPTED
            * `Declined` - DECLINED
        title (Union[Unset, str]):
        description (Union[Unset, TaskCreateDescription]):
        assignee_duids (Union[Unset, List[str]]):
        subscriber_duids (Union[Unset, List[str]]):
        tag_duids (Union[Unset, List[str]]):
        priority (Union[Unset, None, Priority]): * `Critical` - CRITICAL
            * `High` - HIGH
            * `Medium` - MEDIUM
            * `Low` - LOW
        size (Union[Unset, None, int]):
        due_at (Union[Unset, None, datetime.datetime]):
        remind_at (Union[Unset, None, datetime.datetime]):
    """

    duid: str
    dartboard_duid: str
    order: str
    status_duid: str
    source_user: Union[Unset, None, str] = UNSET
    source_type: Union[Unset, TaskSourceType] = TaskSourceType.UNKNOWN
    drafter_duid: Union[Unset, None, str] = UNSET
    in_trash: Union[Unset, bool] = UNSET
    recommendation_status: Union[Unset, None, RecommendationStatus] = UNSET
    title: Union[Unset, str] = UNSET
    description: Union[Unset, "TaskCreateDescription"] = UNSET
    assignee_duids: Union[Unset, List[str]] = UNSET
    subscriber_duids: Union[Unset, List[str]] = UNSET
    tag_duids: Union[Unset, List[str]] = UNSET
    priority: Union[Unset, None, Priority] = UNSET
    size: Union[Unset, None, int] = UNSET
    due_at: Union[Unset, None, datetime.datetime] = UNSET
    remind_at: Union[Unset, None, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid
        dartboard_duid = self.dartboard_duid
        order = self.order
        status_duid = self.status_duid
        source_user = self.source_user
        source_type: Union[Unset, str] = UNSET
        if not isinstance(self.source_type, Unset):
            source_type = self.source_type.value

        drafter_duid = self.drafter_duid
        in_trash = self.in_trash
        recommendation_status: Union[Unset, None, str] = UNSET
        if not isinstance(self.recommendation_status, Unset):
            recommendation_status = self.recommendation_status.value if self.recommendation_status else None

        title = self.title
        description: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.description, Unset):
            description = self.description.to_dict()

        assignee_duids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.assignee_duids, Unset):
            assignee_duids = self.assignee_duids

        subscriber_duids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.subscriber_duids, Unset):
            subscriber_duids = self.subscriber_duids

        tag_duids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tag_duids, Unset):
            tag_duids = self.tag_duids

        priority: Union[Unset, None, str] = UNSET
        if not isinstance(self.priority, Unset):
            priority = self.priority.value if self.priority else None

        size = self.size
        due_at: Union[Unset, None, str] = UNSET
        if not isinstance(self.due_at, Unset):
            due_at = self.due_at.isoformat() if self.due_at else None

        remind_at: Union[Unset, None, str] = UNSET
        if not isinstance(self.remind_at, Unset):
            remind_at = self.remind_at.isoformat() if self.remind_at else None

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "dartboardDuid": dartboard_duid,
                "order": order,
                "statusDuid": status_duid,
            }
        )
        if source_user is not UNSET:
            field_dict["sourceUser"] = source_user
        if source_type is not UNSET:
            field_dict["sourceType"] = source_type
        if drafter_duid is not UNSET:
            field_dict["drafterDuid"] = drafter_duid
        if in_trash is not UNSET:
            field_dict["inTrash"] = in_trash
        if recommendation_status is not UNSET:
            field_dict["recommendationStatus"] = recommendation_status
        if title is not UNSET:
            field_dict["title"] = title
        if description is not UNSET:
            field_dict["description"] = description
        if assignee_duids is not UNSET:
            field_dict["assigneeDuids"] = assignee_duids
        if subscriber_duids is not UNSET:
            field_dict["subscriberDuids"] = subscriber_duids
        if tag_duids is not UNSET:
            field_dict["tagDuids"] = tag_duids
        if priority is not UNSET:
            field_dict["priority"] = priority
        if size is not UNSET:
            field_dict["size"] = size
        if due_at is not UNSET:
            field_dict["dueAt"] = due_at
        if remind_at is not UNSET:
            field_dict["remindAt"] = remind_at

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.task_create_description import TaskCreateDescription

        d = src_dict.copy()
        duid = d.pop("duid")

        dartboard_duid = d.pop("dartboardDuid")

        order = d.pop("order")

        status_duid = d.pop("statusDuid")

        source_user = d.pop("sourceUser", UNSET)

        _source_type = d.pop("sourceType", UNSET)
        source_type: Union[Unset, TaskSourceType]
        if isinstance(_source_type, Unset):
            source_type = UNSET
        else:
            source_type = TaskSourceType(_source_type)

        drafter_duid = d.pop("drafterDuid", UNSET)

        in_trash = d.pop("inTrash", UNSET)

        _recommendation_status = d.pop("recommendationStatus", UNSET)
        recommendation_status: Union[Unset, None, RecommendationStatus]
        if _recommendation_status is None:
            recommendation_status = None
        elif isinstance(_recommendation_status, Unset):
            recommendation_status = UNSET
        else:
            recommendation_status = RecommendationStatus(_recommendation_status)

        title = d.pop("title", UNSET)

        _description = d.pop("description", UNSET)
        description: Union[Unset, TaskCreateDescription]
        if isinstance(_description, Unset):
            description = UNSET
        else:
            description = TaskCreateDescription.from_dict(_description)

        assignee_duids = cast(List[str], d.pop("assigneeDuids", UNSET))

        subscriber_duids = cast(List[str], d.pop("subscriberDuids", UNSET))

        tag_duids = cast(List[str], d.pop("tagDuids", UNSET))

        _priority = d.pop("priority", UNSET)
        priority: Union[Unset, None, Priority]
        if _priority is None:
            priority = None
        elif isinstance(_priority, Unset):
            priority = UNSET
        else:
            priority = Priority(_priority)

        size = d.pop("size", UNSET)

        _due_at = d.pop("dueAt", UNSET)
        due_at: Union[Unset, None, datetime.datetime]
        if _due_at is None:
            due_at = None
        elif isinstance(_due_at, Unset):
            due_at = UNSET
        else:
            due_at = isoparse(_due_at)

        _remind_at = d.pop("remindAt", UNSET)
        remind_at: Union[Unset, None, datetime.datetime]
        if _remind_at is None:
            remind_at = None
        elif isinstance(_remind_at, Unset):
            remind_at = UNSET
        else:
            remind_at = isoparse(_remind_at)

        task_create = cls(
            duid=duid,
            dartboard_duid=dartboard_duid,
            order=order,
            status_duid=status_duid,
            source_user=source_user,
            source_type=source_type,
            drafter_duid=drafter_duid,
            in_trash=in_trash,
            recommendation_status=recommendation_status,
            title=title,
            description=description,
            assignee_duids=assignee_duids,
            subscriber_duids=subscriber_duids,
            tag_duids=tag_duids,
            priority=priority,
            size=size,
            due_at=due_at,
            remind_at=remind_at,
        )

        task_create.additional_properties = d
        return task_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="TaskDocRelationshipCreate")


@_attrs_define
class TaskDocRelationshipCreate:
    """
    Attributes:
        duid (str):
        task_duid (str):
        doc_duid (str):
    """

    duid: str
    task_duid: str
    doc_duid: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid
        task_duid = self.task_duid
        doc_duid = self.doc_duid

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "taskDuid": task_duid,
                "docDuid": doc_duid,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        duid = d.pop("duid")

        task_duid = d.pop("taskDuid")

        doc_duid = d.pop("docDuid")

        task_doc_relationship_create = cls(
            duid=duid,
            task_duid=task_duid,
            doc_duid=doc_duid,
        )

        task_doc_relationship_create.additional_properties = d
        return task_doc_relationship_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Optional, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.color_name import ColorName
from ..models.dartboard_kind import DartboardKind
from ..models.icon_kind import IconKind
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.user_dartboard_layout import UserDartboardLayout


T = TypeVar("T", bound="Dartboard")


@_attrs_define
class Dartboard:
    """
    Attributes:
        duid (str):
        space_duid (str):
        kind (DartboardKind): * `Active` - ACTIVE
            * `Next` - NEXT
            * `Backlog` - BACKLOG
            * `YC` - YC
            * `Finished` - FINISHED
            * `Custom` - CUSTOM
        order (str):
        title (str):
        description (str):
        icon_kind (IconKind): * `None` - NONE
            * `Icon` - ICON
            * `Emoji` - EMOJI
        icon_name_or_emoji (str):
        color_name (ColorName): * `Red` - RED
            * `Dark Blue` - DARK_BLUE
            * `Dark Orange` - DARK_ORANGE
            * `Dark Green` - DARK_GREEN
            * `Purple` - PURPLE
            * `Dark Teal` - DARK_TEAL
            * `Pink` - PINK
            * `Orange` - ORANGE
            * `Green` - GREEN
            * `Yellow` - YELLOW
            * `Brown` - BROWN
            * `Dark Red` - DARK_RED
            * `Flat Green` - FLAT_GREEN
            * `Red Orange` - RED_ORANGE
            * `Teal` - TEAL
            * `Light Green` - LIGHT_GREEN
            * `Light Blue` - LIGHT_BLUE
            * `Light Purple` - LIGHT_PURPLE
            * `Light Orange` - LIGHT_ORANGE
            * `Light Pink` - LIGHT_PINK
            * `Tan` - TAN
            * `Dark Gray` - DARK_GRAY
            * `Light Brown` - LIGHT_BROWN
            * `Light Gray` - LIGHT_GRAY
        user_duids_to_layout_duids (List['UserDartboardLayout']):
        updated_by_client_duid (Union[Unset, None, str]):
        index (Optional[int]):
        started_at (Optional[datetime.datetime]):
        finished_at (Optional[datetime.datetime]):
    """

    duid: str
    space_duid: str
    kind: DartboardKind
    order: str
    title: str
    description: str
    icon_kind: IconKind
    icon_name_or_emoji: str
    color_name: ColorName
    user_duids_to_layout_duids: List["UserDartboardLayout"]
    index: Optional[int]
    started_at: Optional[datetime.datetime]
    finished_at: Optional[datetime.datetime]
    updated_by_client_duid: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid
        space_duid = self.space_duid
        kind = self.kind.value

        order = self.order
        title = self.title
        description = self.description
        icon_kind = self.icon_kind.value

        icon_name_or_emoji = self.icon_name_or_emoji
        color_name = self.color_name.value

        user_duids_to_layout_duids = []
        for user_duids_to_layout_duids_item_data in self.user_duids_to_layout_duids:
            user_duids_to_layout_duids_item = user_duids_to_layout_duids_item_data.to_dict()

            user_duids_to_layout_duids.append(user_duids_to_layout_duids_item)

        updated_by_client_duid = self.updated_by_client_duid
        index = self.index
        started_at = self.started_at.isoformat() if self.started_at else None

        finished_at = self.finished_at.isoformat() if self.finished_at else None

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "spaceDuid": space_duid,
                "kind": kind,
                "order": order,
                "title": title,
                "description": description,
                "iconKind": icon_kind,
                "iconNameOrEmoji": icon_name_or_emoji,
                "colorName": color_name,
                "userDuidsToLayoutDuids": user_duids_to_layout_duids,
                "index": index,
                "startedAt": started_at,
                "finishedAt": finished_at,
            }
        )
        if updated_by_client_duid is not UNSET:
            field_dict["updatedByClientDuid"] = updated_by_client_duid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.user_dartboard_layout import UserDartboardLayout

        d = src_dict.copy()
        duid = d.pop("duid")

        space_duid = d.pop("spaceDuid")

        kind = DartboardKind(d.pop("kind"))

        order = d.pop("order")

        title = d.pop("title")

        description = d.pop("description")

        icon_kind = IconKind(d.pop("iconKind"))

        icon_name_or_emoji = d.pop("iconNameOrEmoji")

        color_name = ColorName(d.pop("colorName"))

        user_duids_to_layout_duids = []
        _user_duids_to_layout_duids = d.pop("userDuidsToLayoutDuids")
        for user_duids_to_layout_duids_item_data in _user_duids_to_layout_duids:
            user_duids_to_layout_duids_item = UserDartboardLayout.from_dict(user_duids_to_layout_duids_item_data)

            user_duids_to_layout_duids.append(user_duids_to_layout_duids_item)

        updated_by_client_duid = d.pop("updatedByClientDuid", UNSET)

        index = d.pop("index")

        _started_at = d.pop("startedAt")
        started_at: Optional[datetime.datetime]
        if _started_at is None:
            started_at = None
        else:
            started_at = isoparse(_started_at)

        _finished_at = d.pop("finishedAt")
        finished_at: Optional[datetime.datetime]
        if _finished_at is None:
            finished_at = None
        else:
            finished_at = isoparse(_finished_at)

        dartboard = cls(
            duid=duid,
            space_duid=space_duid,
            kind=kind,
            order=order,
            title=title,
            description=description,
            icon_kind=icon_kind,
            icon_name_or_emoji=icon_name_or_emoji,
            color_name=color_name,
            user_duids_to_layout_duids=user_duids_to_layout_duids,
            updated_by_client_duid=updated_by_client_duid,
            index=index,
            started_at=started_at,
            finished_at=finished_at,
        )

        dartboard.additional_properties = d
        return dartboard

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class TaskLinkKind(str, Enum):
    GITHUB_BRANCH = "GitHub Branch"
    GITHUB_EXPANSION = "GitHub Expansion"
    GITHUB_PULL_REQUEST = "GitHub Pull Request"
    NOTION_DOCUMENT = "Notion Document"
    NOTION_DOCUMENT_DOESNT_EXIST = "Notion Document Doesnt Exist"
    NOTION_DOCUMENT_PARSE_FAILED = "Notion Document Parse Failed"
    NOTION_EXPANSION = "Notion Expansion"
    NOTION_LINK = "Notion Link"
    SLACK_EXPANSION = "Slack Expansion"
    SOURCE_FROM_TEMPLATE = "Source From Template"
    SOURCE_FROM_THIRD_PARTY = "Source From Third Party"
    STANDARD = "Standard"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="RelationshipCreate")


@_attrs_define
class RelationshipCreate:
    """
    Attributes:
        duid (str):
        source_duid (str):
        kind_duid (str):
        target_duid (str):
        is_forward (Union[Unset, bool]):
    """

    duid: str
    source_duid: str
    kind_duid: str
    target_duid: str
    is_forward: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid
        source_duid = self.source_duid
        kind_duid = self.kind_duid
        target_duid = self.target_duid
        is_forward = self.is_forward

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "sourceDuid": source_duid,
                "kindDuid": kind_duid,
                "targetDuid": target_duid,
            }
        )
        if is_forward is not UNSET:
            field_dict["isForward"] = is_forward

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        duid = d.pop("duid")

        source_duid = d.pop("sourceDuid")

        kind_duid = d.pop("kindDuid")

        target_duid = d.pop("targetDuid")

        is_forward = d.pop("isForward", UNSET)

        relationship_create = cls(
            duid=duid,
            source_duid=source_duid,
            kind_duid=kind_duid,
            target_duid=target_duid,
            is_forward=is_forward,
        )

        relationship_create.additional_properties = d
        return relationship_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Optional, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.subscription import Subscription

if TYPE_CHECKING:
    from ..models.discord_integration import DiscordIntegration
    from ..models.github_integration import GithubIntegration
    from ..models.notion_integration import NotionIntegration
    from ..models.slack_integration import SlackIntegration
    from ..models.tenant_entitlement_overrides import TenantEntitlementOverrides
    from ..models.yc_integration import YcIntegration


T = TypeVar("T", bound="Tenant")


@_attrs_define
class Tenant:
    """
    Attributes:
        duid (str):
        is_dart (bool):
        name (str):
        subscription (Subscription): * `Personal` - PERSONAL
            * `Premium` - PREMIUM
        entitlement_overrides (TenantEntitlementOverrides):
        backlog_enabled (bool):
        multiple_assignees_enabled (bool):
        priority_enabled (bool):
        due_date_enabled (bool):
        size_enabled (bool):
        copy_parent_fields_on_create (bool):
        webhook_enabled (bool):
        webhook_secret (str):
        image_url (Optional[str]):
        webhook_url (Optional[str]):
        yc (Optional[YcIntegration]):
        notion_integration (Optional[NotionIntegration]):
        slack_integration (Optional[SlackIntegration]):
        discord_integration (Optional[DiscordIntegration]):
        github_integration (Optional[GithubIntegration]):
    """

    duid: str
    is_dart: bool
    name: str
    subscription: Subscription
    entitlement_overrides: "TenantEntitlementOverrides"
    backlog_enabled: bool
    multiple_assignees_enabled: bool
    priority_enabled: bool
    due_date_enabled: bool
    size_enabled: bool
    copy_parent_fields_on_create: bool
    webhook_enabled: bool
    webhook_secret: str
    image_url: Optional[str]
    webhook_url: Optional[str]
    yc: Optional["YcIntegration"]
    notion_integration: Optional["NotionIntegration"]
    slack_integration: Optional["SlackIntegration"]
    discord_integration: Optional["DiscordIntegration"]
    github_integration: Optional["GithubIntegration"]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid
        is_dart = self.is_dart
        name = self.name
        subscription = self.subscription.value

        entitlement_overrides = self.entitlement_overrides.to_dict()

        backlog_enabled = self.backlog_enabled
        multiple_assignees_enabled = self.multiple_assignees_enabled
        priority_enabled = self.priority_enabled
        due_date_enabled = self.due_date_enabled
        size_enabled = self.size_enabled
        copy_parent_fields_on_create = self.copy_parent_fields_on_create
        webhook_enabled = self.webhook_enabled
        webhook_secret = self.webhook_secret
        image_url = self.image_url
        webhook_url = self.webhook_url
        yc = self.yc.to_dict() if self.yc else None

        notion_integration = self.notion_integration.to_dict() if self.notion_integration else None

        slack_integration = self.slack_integration.to_dict() if self.slack_integration else None

        discord_integration = self.discord_integration.to_dict() if self.discord_integration else None

        github_integration = self.github_integration.to_dict() if self.github_integration else None

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "isDart": is_dart,
                "name": name,
                "subscription": subscription,
                "entitlementOverrides": entitlement_overrides,
                "backlogEnabled": backlog_enabled,
                "multipleAssigneesEnabled": multiple_assignees_enabled,
                "priorityEnabled": priority_enabled,
                "dueDateEnabled": due_date_enabled,
                "sizeEnabled": size_enabled,
                "copyParentFieldsOnCreate": copy_parent_fields_on_create,
                "webhookEnabled": webhook_enabled,
                "webhookSecret": webhook_secret,
                "imageUrl": image_url,
                "webhookUrl": webhook_url,
                "yc": yc,
                "notionIntegration": notion_integration,
                "slackIntegration": slack_integration,
                "discordIntegration": discord_integration,
                "githubIntegration": github_integration,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.discord_integration import DiscordIntegration
        from ..models.github_integration import GithubIntegration
        from ..models.notion_integration import NotionIntegration
        from ..models.slack_integration import SlackIntegration
        from ..models.tenant_entitlement_overrides import TenantEntitlementOverrides
        from ..models.yc_integration import YcIntegration

        d = src_dict.copy()
        duid = d.pop("duid")

        is_dart = d.pop("isDart")

        name = d.pop("name")

        subscription = Subscription(d.pop("subscription"))

        entitlement_overrides = TenantEntitlementOverrides.from_dict(d.pop("entitlementOverrides"))

        backlog_enabled = d.pop("backlogEnabled")

        multiple_assignees_enabled = d.pop("multipleAssigneesEnabled")

        priority_enabled = d.pop("priorityEnabled")

        due_date_enabled = d.pop("dueDateEnabled")

        size_enabled = d.pop("sizeEnabled")

        copy_parent_fields_on_create = d.pop("copyParentFieldsOnCreate")

        webhook_enabled = d.pop("webhookEnabled")

        webhook_secret = d.pop("webhookSecret")

        image_url = d.pop("imageUrl")

        webhook_url = d.pop("webhookUrl")

        _yc = d.pop("yc")
        yc: Optional[YcIntegration]
        if _yc is None:
            yc = None
        else:
            yc = YcIntegration.from_dict(_yc)

        _notion_integration = d.pop("notionIntegration")
        notion_integration: Optional[NotionIntegration]
        if _notion_integration is None:
            notion_integration = None
        else:
            notion_integration = NotionIntegration.from_dict(_notion_integration)

        _slack_integration = d.pop("slackIntegration")
        slack_integration: Optional[SlackIntegration]
        if _slack_integration is None:
            slack_integration = None
        else:
            slack_integration = SlackIntegration.from_dict(_slack_integration)

        _discord_integration = d.pop("discordIntegration")
        discord_integration: Optional[DiscordIntegration]
        if _discord_integration is None:
            discord_integration = None
        else:
            discord_integration = DiscordIntegration.from_dict(_discord_integration)

        _github_integration = d.pop("githubIntegration")
        github_integration: Optional[GithubIntegration]
        if _github_integration is None:
            github_integration = None
        else:
            github_integration = GithubIntegration.from_dict(_github_integration)

        tenant = cls(
            duid=duid,
            is_dart=is_dart,
            name=name,
            subscription=subscription,
            entitlement_overrides=entitlement_overrides,
            backlog_enabled=backlog_enabled,
            multiple_assignees_enabled=multiple_assignees_enabled,
            priority_enabled=priority_enabled,
            due_date_enabled=due_date_enabled,
            size_enabled=size_enabled,
            copy_parent_fields_on_create=copy_parent_fields_on_create,
            webhook_enabled=webhook_enabled,
            webhook_secret=webhook_secret,
            image_url=image_url,
            webhook_url=webhook_url,
            yc=yc,
            notion_integration=notion_integration,
            slack_integration=slack_integration,
            discord_integration=discord_integration,
            github_integration=github_integration,
        )

        tenant.additional_properties = d
        return tenant

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="CommentReactionUpdate")


@_attrs_define
class CommentReactionUpdate:
    """
    Attributes:
        duid (str):
        author_duid (Union[Unset, str]):
        comment_duid (Union[Unset, str]):
        emoji (Union[Unset, str]):
    """

    duid: str
    author_duid: Union[Unset, str] = UNSET
    comment_duid: Union[Unset, str] = UNSET
    emoji: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid
        author_duid = self.author_duid
        comment_duid = self.comment_duid
        emoji = self.emoji

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
            }
        )
        if author_duid is not UNSET:
            field_dict["authorDuid"] = author_duid
        if comment_duid is not UNSET:
            field_dict["commentDuid"] = comment_duid
        if emoji is not UNSET:
            field_dict["emoji"] = emoji

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        duid = d.pop("duid")

        author_duid = d.pop("authorDuid", UNSET)

        comment_duid = d.pop("commentDuid", UNSET)

        emoji = d.pop("emoji", UNSET)

        comment_reaction_update = cls(
            duid=duid,
            author_duid=author_duid,
            comment_duid=comment_duid,
            emoji=emoji,
        )

        comment_reaction_update.additional_properties = d
        return comment_reaction_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DartboardKind(str, Enum):
    ACTIVE = "Active"
    BACKLOG = "Backlog"
    CUSTOM = "Custom"
    FINISHED = "Finished"
    NEXT = "Next"
    YC = "YC"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.color_name import ColorName
from ..models.doc_source_type import DocSourceType
from ..models.icon_kind import IconKind
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.doc_update_text import DocUpdateText


T = TypeVar("T", bound="DocUpdate")


@_attrs_define
class DocUpdate:
    """
    Attributes:
        duid (str):
        source_user (Union[Unset, None, str]):
        source_type (Union[Unset, DocSourceType]): * `Unknown` - UNKNOWN
            * `Recommendation` - RECOMMENDATION
            * `Application` - APPLICATION Default: DocSourceType.UNKNOWN.
        drafter_duid (Union[Unset, None, str]):
        in_trash (Union[Unset, bool]):
        folder_duid (Union[Unset, str]):
        order (Union[Unset, str]):
        title (Union[Unset, str]):
        text (Union[Unset, DocUpdateText]):
        editor_duids (Union[Unset, List[str]]):
        subscriber_duids (Union[Unset, List[str]]):
        icon_kind (Union[Unset, IconKind]): * `None` - NONE
            * `Icon` - ICON
            * `Emoji` - EMOJI
        icon_name_or_emoji (Union[Unset, str]):
        color_name (Union[Unset, ColorName]): * `Red` - RED
            * `Dark Blue` - DARK_BLUE
            * `Dark Orange` - DARK_ORANGE
            * `Dark Green` - DARK_GREEN
            * `Purple` - PURPLE
            * `Dark Teal` - DARK_TEAL
            * `Pink` - PINK
            * `Orange` - ORANGE
            * `Green` - GREEN
            * `Yellow` - YELLOW
            * `Brown` - BROWN
            * `Dark Red` - DARK_RED
            * `Flat Green` - FLAT_GREEN
            * `Red Orange` - RED_ORANGE
            * `Teal` - TEAL
            * `Light Green` - LIGHT_GREEN
            * `Light Blue` - LIGHT_BLUE
            * `Light Purple` - LIGHT_PURPLE
            * `Light Orange` - LIGHT_ORANGE
            * `Light Pink` - LIGHT_PINK
            * `Tan` - TAN
            * `Dark Gray` - DARK_GRAY
            * `Light Brown` - LIGHT_BROWN
            * `Light Gray` - LIGHT_GRAY
    """

    duid: str
    source_user: Union[Unset, None, str] = UNSET
    source_type: Union[Unset, DocSourceType] = DocSourceType.UNKNOWN
    drafter_duid: Union[Unset, None, str] = UNSET
    in_trash: Union[Unset, bool] = UNSET
    folder_duid: Union[Unset, str] = UNSET
    order: Union[Unset, str] = UNSET
    title: Union[Unset, str] = UNSET
    text: Union[Unset, "DocUpdateText"] = UNSET
    editor_duids: Union[Unset, List[str]] = UNSET
    subscriber_duids: Union[Unset, List[str]] = UNSET
    icon_kind: Union[Unset, IconKind] = UNSET
    icon_name_or_emoji: Union[Unset, str] = UNSET
    color_name: Union[Unset, ColorName] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid
        source_user = self.source_user
        source_type: Union[Unset, str] = UNSET
        if not isinstance(self.source_type, Unset):
            source_type = self.source_type.value

        drafter_duid = self.drafter_duid
        in_trash = self.in_trash
        folder_duid = self.folder_duid
        order = self.order
        title = self.title
        text: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.text, Unset):
            text = self.text.to_dict()

        editor_duids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.editor_duids, Unset):
            editor_duids = self.editor_duids

        subscriber_duids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.subscriber_duids, Unset):
            subscriber_duids = self.subscriber_duids

        icon_kind: Union[Unset, str] = UNSET
        if not isinstance(self.icon_kind, Unset):
            icon_kind = self.icon_kind.value

        icon_name_or_emoji = self.icon_name_or_emoji
        color_name: Union[Unset, str] = UNSET
        if not isinstance(self.color_name, Unset):
            color_name = self.color_name.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
            }
        )
        if source_user is not UNSET:
            field_dict["sourceUser"] = source_user
        if source_type is not UNSET:
            field_dict["sourceType"] = source_type
        if drafter_duid is not UNSET:
            field_dict["drafterDuid"] = drafter_duid
        if in_trash is not UNSET:
            field_dict["inTrash"] = in_trash
        if folder_duid is not UNSET:
            field_dict["folderDuid"] = folder_duid
        if order is not UNSET:
            field_dict["order"] = order
        if title is not UNSET:
            field_dict["title"] = title
        if text is not UNSET:
            field_dict["text"] = text
        if editor_duids is not UNSET:
            field_dict["editorDuids"] = editor_duids
        if subscriber_duids is not UNSET:
            field_dict["subscriberDuids"] = subscriber_duids
        if icon_kind is not UNSET:
            field_dict["iconKind"] = icon_kind
        if icon_name_or_emoji is not UNSET:
            field_dict["iconNameOrEmoji"] = icon_name_or_emoji
        if color_name is not UNSET:
            field_dict["colorName"] = color_name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.doc_update_text import DocUpdateText

        d = src_dict.copy()
        duid = d.pop("duid")

        source_user = d.pop("sourceUser", UNSET)

        _source_type = d.pop("sourceType", UNSET)
        source_type: Union[Unset, DocSourceType]
        if isinstance(_source_type, Unset):
            source_type = UNSET
        else:
            source_type = DocSourceType(_source_type)

        drafter_duid = d.pop("drafterDuid", UNSET)

        in_trash = d.pop("inTrash", UNSET)

        folder_duid = d.pop("folderDuid", UNSET)

        order = d.pop("order", UNSET)

        title = d.pop("title", UNSET)

        _text = d.pop("text", UNSET)
        text: Union[Unset, DocUpdateText]
        if isinstance(_text, Unset):
            text = UNSET
        else:
            text = DocUpdateText.from_dict(_text)

        editor_duids = cast(List[str], d.pop("editorDuids", UNSET))

        subscriber_duids = cast(List[str], d.pop("subscriberDuids", UNSET))

        _icon_kind = d.pop("iconKind", UNSET)
        icon_kind: Union[Unset, IconKind]
        if isinstance(_icon_kind, Unset):
            icon_kind = UNSET
        else:
            icon_kind = IconKind(_icon_kind)

        icon_name_or_emoji = d.pop("iconNameOrEmoji", UNSET)

        _color_name = d.pop("colorName", UNSET)
        color_name: Union[Unset, ColorName]
        if isinstance(_color_name, Unset):
            color_name = UNSET
        else:
            color_name = ColorName(_color_name)

        doc_update = cls(
            duid=duid,
            source_user=source_user,
            source_type=source_type,
            drafter_duid=drafter_duid,
            in_trash=in_trash,
            folder_duid=folder_duid,
            order=order,
            title=title,
            text=text,
            editor_duids=editor_duids,
            subscriber_duids=subscriber_duids,
            icon_kind=icon_kind,
            icon_name_or_emoji=icon_name_or_emoji,
            color_name=color_name,
        )

        doc_update.additional_properties = d
        return doc_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.filter_applicability import FilterApplicability
from ..models.filter_connector import FilterConnector

T = TypeVar("T", bound="FilterSet")


@_attrs_define
class FilterSet:
    """
    Attributes:
        id (str):
        field (str):
        locked (bool):
        applicability (FilterApplicability): * `is` - IS
            * `is not` - IS_NOT
            * `is not set` - IS_NOT_SET
            * `include` - INCLUDE
            * `don't include` - DO_NOT_INCLUDE
            * `are not set` - ARE_NOT_SET
            * `exists` - EXISTS
            * `exist` - EXIST
            * `contains` - CONTAINS
            * `contain` - CONTAIN
            * `is before` - IS_BEFORE
            * `is after` - IS_AFTER
            * `is between` - IS_BETWEEN
        connector (FilterConnector): * `or` - OR
            * `and` - AND
        values (List[Any]):
    """

    id: str
    field: str
    locked: bool
    applicability: FilterApplicability
    connector: FilterConnector
    values: List[Any]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        field = self.field
        locked = self.locked
        applicability = self.applicability.value

        connector = self.connector.value

        values = self.values

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "field": field,
                "locked": locked,
                "applicability": applicability,
                "connector": connector,
                "values": values,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        field = d.pop("field")

        locked = d.pop("locked")

        applicability = FilterApplicability(d.pop("applicability"))

        connector = FilterConnector(d.pop("connector"))

        values = cast(List[Any], d.pop("values"))

        filter_set = cls(
            id=id,
            field=field,
            locked=locked,
            applicability=applicability,
            connector=connector,
            values=values,
        )

        filter_set.additional_properties = d
        return filter_set

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.event_kind import EventKind
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.event_update_adtl import EventUpdateAdtl


T = TypeVar("T", bound="EventUpdate")


@_attrs_define
class EventUpdate:
    """
    Attributes:
        kind (Union[Unset, EventKind]): * `tasks/create` - TASK_CREATE
            * `tasks/update_assignees` - TASK_UPDATE_ASSIGNEES
            * `tasks/update_status` - TASK_UPDATE_STATUS
            * `tasks/update_title` - TASK_UPDATE_TITLE
            * `tasks/update_description` - TASK_UPDATE_DESCRIPTION
            * `tasks/set_relationship` - TASK_SET_RELATIONSHIP
            * `tasks/remove_relationship` - TASK_REMOVE_RELATIONSHIP
            * `tasks/update_subscriptions` - TASK_UPDATE_SUBSCRIPTIONS
            * `tasks/update_other` - TASK_UPDATE_OTHER
            * `tasks/comment` - TASK_COMMENT
            * `tasks/due_date_tomorrow` - TASK_DUE_DATE_TOMORROW
            * `tasks/reminder_now` - TASK_REMINDER_NOW
            * `tasks/delete` - TASK_DELETE
            * `tasks/delete_fully` - TASK_DELETE_FULLY
            * `docs/create` - DOC_CREATE
            * `docs/update_title` - DOC_UPDATE_TITLE
            * `docs/update_other` - DOC_UPDATE_OTHER
            * `docs/delete` - DOC_DELETE
            * `pages/create` - PAGE_CREATE
            * `pages/update_title` - PAGE_UPDATE_TITLE
            * `pages/update_permissions` - PAGE_UPDATE_PERMISSIONS
            * `pages/update_other` - PAGE_UPDATE_OTHER
            * `pages/rollover` - DARTBOARD_ROLLOVER
            * `pages/delete` - PAGE_DELETE
            * `workspace/invite` - WORKSPACE_INVITE
            * `workspace/join` - WORKSPACE_JOIN
            * `workspace/leave` - WORKSPACE_LEAVE
            * `workspace/update_status` - WORKSPACE_UPDATE_STATUS
            * `workspace/update_other` - WORKSPACE_UPDATE_OTHER
            * `workspace/create` - WORKSPACE_CREATE
            * `workspace/data_import` - WORKSPACE_DATA_IMPORT
            * `workspace/data_export` - WORKSPACE_DATA_EXPORT
            * `workspace/delete` - WORKSPACE_DELETE
            * `workspace/upgrade` - WORKSPACE_UPGRADE
            * `workspace/downgrade_initialize` - WORKSPACE_DOWNGRADE_INITIALIZE
            * `workspace/downgrade_finalize` - WORKSPACE_DOWNGRADE_FINALIZE
            * `workspace/become_active` - WORKSPACE_BECOME_ACTIVE
            * `workspace/become_inactive` - WORKSPACE_BECOME_INACTIVE
            * `onboarding/finish_step` - ONBOARDING_FINISH_STEP
            * `profile/update` - PROFILE_UPDATE
            * `profile/delete` - PROFILE_DELETE
            * `profile/become_active` - PROFILE_BECOME_ACTIVE
            * `profile/become_inactive` - PROFILE_BECOME_INACTIVE
            * `ai/props` - AI_PROPS
            * `ai/subtasks` - AI_SUBTASKS
            * `ai/content` - AI_CONTENT
            * `ai/translate` - AI_TRANSLATE
            * `ai/emoji` - AI_EMOJI
            * `ai/feedback` - AI_FEEDBACK
            * `ai/icon` - AI_ICON
            * `help/resource_click` - HELP_RESOURCE_CLICK
            * `usage/submit_feedback` - USAGE_SUBMIT_FEEDBACK
            * `usage/undo` - USAGE_UNDO
            * `usage/redo` - USAGE_REDO
            * `usage/open_command_center` - USAGE_OPEN_COMMAND_CENTER
            * `usage/open_rightbar` - USAGE_OPEN_RIGHTBAR
            * `usage/open_fullscreen` - USAGE_OPEN_FULLSCREEN
            * `usage/copy_task_link` - USAGE_COPY_TASK_LINK
            * `usage/copy_branch` - USAGE_COPY_BRANCH
            * `usage/open_search` - USAGE_OPEN_SEARCH
            * `usage/nlp_raw_create` - USAGE_NLP_RAW_CREATE
            * `usage/nlp_raw_delete` - USAGE_NLP_RAW_DELETE
            * `usage/nlp_typeahead_open` - USAGE_NLP_TYPEAHEAD_OPEN
            * `usage/nlp_typeahead_accept` - USAGE_NLP_TYPEAHEAD_ACCEPT
        actor_duid (Union[Unset, str]):
        adtl (Union[Unset, EventUpdateAdtl]):
    """

    kind: Union[Unset, EventKind] = UNSET
    actor_duid: Union[Unset, str] = UNSET
    adtl: Union[Unset, "EventUpdateAdtl"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        kind: Union[Unset, str] = UNSET
        if not isinstance(self.kind, Unset):
            kind = self.kind.value

        actor_duid = self.actor_duid
        adtl: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.adtl, Unset):
            adtl = self.adtl.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if kind is not UNSET:
            field_dict["kind"] = kind
        if actor_duid is not UNSET:
            field_dict["actorDuid"] = actor_duid
        if adtl is not UNSET:
            field_dict["adtl"] = adtl

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.event_update_adtl import EventUpdateAdtl

        d = src_dict.copy()
        _kind = d.pop("kind", UNSET)
        kind: Union[Unset, EventKind]
        if isinstance(_kind, Unset):
            kind = UNSET
        else:
            kind = EventKind(_kind)

        actor_duid = d.pop("actorDuid", UNSET)

        _adtl = d.pop("adtl", UNSET)
        adtl: Union[Unset, EventUpdateAdtl]
        if isinstance(_adtl, Unset):
            adtl = UNSET
        else:
            adtl = EventUpdateAdtl.from_dict(_adtl)

        event_update = cls(
            kind=kind,
            actor_duid=actor_duid,
            adtl=adtl,
        )

        event_update.additional_properties = d
        return event_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class TaskSourceType(str, Enum):
    API = "API"
    APPBOARD = "AppBoard"
    APPENTER = "AppEnter"
    APPLICATION = "Application"
    APPPASTE = "AppPaste"
    APPQUICKADD = "AppQuickAdd"
    APPRELATIONSHIP = "AppRelationship"
    APPREPLICATE = "AppReplicate"
    APPSUBTASK = "AppSubtask"
    APPTCM = "AppTcm"
    CHATGPT = "ChatGPT"
    CLI = "CLI"
    EMAIL = "Email"
    IMPORT = "Import"
    ONBOARDING = "Onboarding"
    RECOMMENDATION = "Recommendation"
    SLACK = "Slack"
    TEMPLATE = "Template"
    UNKNOWN = "Unknown"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class SummaryStatisticKind(str, Enum):
    COMPLETEDPERCENT = "CompletedPercent"
    COMPLETEDPERCENTPOINTS = "CompletedPercentPoints"
    INCOMPLETECOUNT = "IncompleteCount"
    INCOMPLETECOUNTPOINTS = "IncompleteCountPoints"
    NONE = "None"
    TOTALCOUNT = "TotalCount"
    TOTALCOUNTPOINTS = "TotalCountPoints"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.color_name import ColorName
from ..types import UNSET, Unset

T = TypeVar("T", bound="Tag")


@_attrs_define
class Tag:
    """
    Attributes:
        duid (str):
        title (str):
        color_name (ColorName): * `Red` - RED
            * `Dark Blue` - DARK_BLUE
            * `Dark Orange` - DARK_ORANGE
            * `Dark Green` - DARK_GREEN
            * `Purple` - PURPLE
            * `Dark Teal` - DARK_TEAL
            * `Pink` - PINK
            * `Orange` - ORANGE
            * `Green` - GREEN
            * `Yellow` - YELLOW
            * `Brown` - BROWN
            * `Dark Red` - DARK_RED
            * `Flat Green` - FLAT_GREEN
            * `Red Orange` - RED_ORANGE
            * `Teal` - TEAL
            * `Light Green` - LIGHT_GREEN
            * `Light Blue` - LIGHT_BLUE
            * `Light Purple` - LIGHT_PURPLE
            * `Light Orange` - LIGHT_ORANGE
            * `Light Pink` - LIGHT_PINK
            * `Tan` - TAN
            * `Dark Gray` - DARK_GRAY
            * `Light Brown` - LIGHT_BROWN
            * `Light Gray` - LIGHT_GRAY
        updated_by_client_duid (Union[Unset, None, str]):
    """

    duid: str
    title: str
    color_name: ColorName
    updated_by_client_duid: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid
        title = self.title
        color_name = self.color_name.value

        updated_by_client_duid = self.updated_by_client_duid

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "title": title,
                "colorName": color_name,
            }
        )
        if updated_by_client_duid is not UNSET:
            field_dict["updatedByClientDuid"] = updated_by_client_duid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        duid = d.pop("duid")

        title = d.pop("title")

        color_name = ColorName(d.pop("colorName"))

        updated_by_client_duid = d.pop("updatedByClientDuid", UNSET)

        tag = cls(
            duid=duid,
            title=title,
            color_name=color_name,
            updated_by_client_duid=updated_by_client_duid,
        )

        tag.additional_properties = d
        return tag

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.event_kind import EventKind

if TYPE_CHECKING:
    from ..models.event_adtl import EventAdtl


T = TypeVar("T", bound="Event")


@_attrs_define
class Event:
    """
    Attributes:
        kind (EventKind): * `tasks/create` - TASK_CREATE
            * `tasks/update_assignees` - TASK_UPDATE_ASSIGNEES
            * `tasks/update_status` - TASK_UPDATE_STATUS
            * `tasks/update_title` - TASK_UPDATE_TITLE
            * `tasks/update_description` - TASK_UPDATE_DESCRIPTION
            * `tasks/set_relationship` - TASK_SET_RELATIONSHIP
            * `tasks/remove_relationship` - TASK_REMOVE_RELATIONSHIP
            * `tasks/update_subscriptions` - TASK_UPDATE_SUBSCRIPTIONS
            * `tasks/update_other` - TASK_UPDATE_OTHER
            * `tasks/comment` - TASK_COMMENT
            * `tasks/due_date_tomorrow` - TASK_DUE_DATE_TOMORROW
            * `tasks/reminder_now` - TASK_REMINDER_NOW
            * `tasks/delete` - TASK_DELETE
            * `tasks/delete_fully` - TASK_DELETE_FULLY
            * `docs/create` - DOC_CREATE
            * `docs/update_title` - DOC_UPDATE_TITLE
            * `docs/update_other` - DOC_UPDATE_OTHER
            * `docs/delete` - DOC_DELETE
            * `pages/create` - PAGE_CREATE
            * `pages/update_title` - PAGE_UPDATE_TITLE
            * `pages/update_permissions` - PAGE_UPDATE_PERMISSIONS
            * `pages/update_other` - PAGE_UPDATE_OTHER
            * `pages/rollover` - DARTBOARD_ROLLOVER
            * `pages/delete` - PAGE_DELETE
            * `workspace/invite` - WORKSPACE_INVITE
            * `workspace/join` - WORKSPACE_JOIN
            * `workspace/leave` - WORKSPACE_LEAVE
            * `workspace/update_status` - WORKSPACE_UPDATE_STATUS
            * `workspace/update_other` - WORKSPACE_UPDATE_OTHER
            * `workspace/create` - WORKSPACE_CREATE
            * `workspace/data_import` - WORKSPACE_DATA_IMPORT
            * `workspace/data_export` - WORKSPACE_DATA_EXPORT
            * `workspace/delete` - WORKSPACE_DELETE
            * `workspace/upgrade` - WORKSPACE_UPGRADE
            * `workspace/downgrade_initialize` - WORKSPACE_DOWNGRADE_INITIALIZE
            * `workspace/downgrade_finalize` - WORKSPACE_DOWNGRADE_FINALIZE
            * `workspace/become_active` - WORKSPACE_BECOME_ACTIVE
            * `workspace/become_inactive` - WORKSPACE_BECOME_INACTIVE
            * `onboarding/finish_step` - ONBOARDING_FINISH_STEP
            * `profile/update` - PROFILE_UPDATE
            * `profile/delete` - PROFILE_DELETE
            * `profile/become_active` - PROFILE_BECOME_ACTIVE
            * `profile/become_inactive` - PROFILE_BECOME_INACTIVE
            * `ai/props` - AI_PROPS
            * `ai/subtasks` - AI_SUBTASKS
            * `ai/content` - AI_CONTENT
            * `ai/translate` - AI_TRANSLATE
            * `ai/emoji` - AI_EMOJI
            * `ai/feedback` - AI_FEEDBACK
            * `ai/icon` - AI_ICON
            * `help/resource_click` - HELP_RESOURCE_CLICK
            * `usage/submit_feedback` - USAGE_SUBMIT_FEEDBACK
            * `usage/undo` - USAGE_UNDO
            * `usage/redo` - USAGE_REDO
            * `usage/open_command_center` - USAGE_OPEN_COMMAND_CENTER
            * `usage/open_rightbar` - USAGE_OPEN_RIGHTBAR
            * `usage/open_fullscreen` - USAGE_OPEN_FULLSCREEN
            * `usage/copy_task_link` - USAGE_COPY_TASK_LINK
            * `usage/copy_branch` - USAGE_COPY_BRANCH
            * `usage/open_search` - USAGE_OPEN_SEARCH
            * `usage/nlp_raw_create` - USAGE_NLP_RAW_CREATE
            * `usage/nlp_raw_delete` - USAGE_NLP_RAW_DELETE
            * `usage/nlp_typeahead_open` - USAGE_NLP_TYPEAHEAD_OPEN
            * `usage/nlp_typeahead_accept` - USAGE_NLP_TYPEAHEAD_ACCEPT
        adtl (EventAdtl):
    """

    kind: EventKind
    adtl: "EventAdtl"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        kind = self.kind.value

        adtl = self.adtl.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "kind": kind,
                "adtl": adtl,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.event_adtl import EventAdtl

        d = src_dict.copy()
        kind = EventKind(d.pop("kind"))

        adtl = EventAdtl.from_dict(d.pop("adtl"))

        event = cls(
            kind=kind,
            adtl=adtl,
        )

        event.additional_properties = d
        return event

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Optional, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.notion_integration_tenant_extension_status import NotionIntegrationTenantExtensionStatus

T = TypeVar("T", bound="NotionIntegration")


@_attrs_define
class NotionIntegration:
    """
    Attributes:
        status (NotionIntegrationTenantExtensionStatus): * `Disabled` - DISABLED
            * `Pending` - PENDING
            * `Enabled` - ENABLED
        workspace_name (Optional[str]):
        workspace_icon (Optional[str]):
    """

    status: NotionIntegrationTenantExtensionStatus
    workspace_name: Optional[str]
    workspace_icon: Optional[str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        status = self.status.value

        workspace_name = self.workspace_name
        workspace_icon = self.workspace_icon

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "status": status,
                "workspaceName": workspace_name,
                "workspaceIcon": workspace_icon,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        status = NotionIntegrationTenantExtensionStatus(d.pop("status"))

        workspace_name = d.pop("workspaceName")

        workspace_icon = d.pop("workspaceIcon")

        notion_integration = cls(
            status=status,
            workspace_name=workspace_name,
            workspace_icon=workspace_icon,
        )

        notion_integration.additional_properties = d
        return notion_integration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="TenantCreate")


@_attrs_define
class TenantCreate:
    """
    Attributes:
        name (str):
        backlog_enabled (Union[Unset, bool]):
        multiple_assignees_enabled (Union[Unset, bool]):
        priority_enabled (Union[Unset, bool]):
        due_date_enabled (Union[Unset, bool]):
        size_enabled (Union[Unset, bool]):
        copy_parent_fields_on_create (Union[Unset, bool]):
        webhook_enabled (Union[Unset, bool]):
        webhook_secret (Union[Unset, str]):
        webhook_url (Union[Unset, None, str]):
    """

    name: str
    backlog_enabled: Union[Unset, bool] = UNSET
    multiple_assignees_enabled: Union[Unset, bool] = UNSET
    priority_enabled: Union[Unset, bool] = UNSET
    due_date_enabled: Union[Unset, bool] = UNSET
    size_enabled: Union[Unset, bool] = UNSET
    copy_parent_fields_on_create: Union[Unset, bool] = UNSET
    webhook_enabled: Union[Unset, bool] = UNSET
    webhook_secret: Union[Unset, str] = UNSET
    webhook_url: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        backlog_enabled = self.backlog_enabled
        multiple_assignees_enabled = self.multiple_assignees_enabled
        priority_enabled = self.priority_enabled
        due_date_enabled = self.due_date_enabled
        size_enabled = self.size_enabled
        copy_parent_fields_on_create = self.copy_parent_fields_on_create
        webhook_enabled = self.webhook_enabled
        webhook_secret = self.webhook_secret
        webhook_url = self.webhook_url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if backlog_enabled is not UNSET:
            field_dict["backlogEnabled"] = backlog_enabled
        if multiple_assignees_enabled is not UNSET:
            field_dict["multipleAssigneesEnabled"] = multiple_assignees_enabled
        if priority_enabled is not UNSET:
            field_dict["priorityEnabled"] = priority_enabled
        if due_date_enabled is not UNSET:
            field_dict["dueDateEnabled"] = due_date_enabled
        if size_enabled is not UNSET:
            field_dict["sizeEnabled"] = size_enabled
        if copy_parent_fields_on_create is not UNSET:
            field_dict["copyParentFieldsOnCreate"] = copy_parent_fields_on_create
        if webhook_enabled is not UNSET:
            field_dict["webhookEnabled"] = webhook_enabled
        if webhook_secret is not UNSET:
            field_dict["webhookSecret"] = webhook_secret
        if webhook_url is not UNSET:
            field_dict["webhookUrl"] = webhook_url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        backlog_enabled = d.pop("backlogEnabled", UNSET)

        multiple_assignees_enabled = d.pop("multipleAssigneesEnabled", UNSET)

        priority_enabled = d.pop("priorityEnabled", UNSET)

        due_date_enabled = d.pop("dueDateEnabled", UNSET)

        size_enabled = d.pop("sizeEnabled", UNSET)

        copy_parent_fields_on_create = d.pop("copyParentFieldsOnCreate", UNSET)

        webhook_enabled = d.pop("webhookEnabled", UNSET)

        webhook_secret = d.pop("webhookSecret", UNSET)

        webhook_url = d.pop("webhookUrl", UNSET)

        tenant_create = cls(
            name=name,
            backlog_enabled=backlog_enabled,
            multiple_assignees_enabled=multiple_assignees_enabled,
            priority_enabled=priority_enabled,
            due_date_enabled=due_date_enabled,
            size_enabled=size_enabled,
            copy_parent_fields_on_create=copy_parent_fields_on_create,
            webhook_enabled=webhook_enabled,
            webhook_secret=webhook_secret,
            webhook_url=webhook_url,
        )

        tenant_create.additional_properties = d
        return tenant_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class OperationModelKind(str, Enum):
    COMMENT = "comment"
    COMMENT_REACTION = "comment_reaction"
    DARTBOARD = "dartboard"
    DOC = "doc"
    EVENT = "event"
    EVENT_SUBSCRIPTION = "event_subscription"
    FOLDER = "folder"
    LAYOUT = "layout"
    RELATIONSHIP = "relationship"
    RELATIONSHIP_KIND = "relationship_kind"
    SPACE = "space"
    STATUS = "status"
    TAG = "tag"
    TASK = "task"
    TASK_ATTACHMENT = "task_attachment"
    TASK_DOC_RELATIONSHIP = "task_doc_relationship"
    TASK_LINK = "task_link"
    TENANT = "tenant"
    USER = "user"
    USER_DARTBOARD_LAYOUT = "user_dartboard_layout"
    VIEW = "view"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.operation_kind import OperationKind
from ..models.operation_model_kind import OperationModelKind

if TYPE_CHECKING:
    from ..models.comment_create import CommentCreate
    from ..models.comment_reaction_create import CommentReactionCreate
    from ..models.comment_reaction_update import CommentReactionUpdate
    from ..models.comment_update import CommentUpdate
    from ..models.dartboard_create import DartboardCreate
    from ..models.dartboard_update import DartboardUpdate
    from ..models.doc_create import DocCreate
    from ..models.doc_update import DocUpdate
    from ..models.event_create import EventCreate
    from ..models.event_subscription_create import EventSubscriptionCreate
    from ..models.event_subscription_update import EventSubscriptionUpdate
    from ..models.event_update import EventUpdate
    from ..models.folder_create import FolderCreate
    from ..models.folder_update import FolderUpdate
    from ..models.layout_create import LayoutCreate
    from ..models.layout_update import LayoutUpdate
    from ..models.relationship_create import RelationshipCreate
    from ..models.relationship_kind_create import RelationshipKindCreate
    from ..models.relationship_kind_update import RelationshipKindUpdate
    from ..models.relationship_update import RelationshipUpdate
    from ..models.space_create import SpaceCreate
    from ..models.space_update import SpaceUpdate
    from ..models.status_create import StatusCreate
    from ..models.status_update import StatusUpdate
    from ..models.tag_create import TagCreate
    from ..models.tag_update import TagUpdate
    from ..models.task_attachment_create import TaskAttachmentCreate
    from ..models.task_attachment_update import TaskAttachmentUpdate
    from ..models.task_create import TaskCreate
    from ..models.task_doc_relationship_create import TaskDocRelationshipCreate
    from ..models.task_doc_relationship_update import TaskDocRelationshipUpdate
    from ..models.task_link_create import TaskLinkCreate
    from ..models.task_link_update import TaskLinkUpdate
    from ..models.task_update import TaskUpdate
    from ..models.tenant_create import TenantCreate
    from ..models.tenant_update import TenantUpdate
    from ..models.user_create import UserCreate
    from ..models.user_dartboard_layout_create import UserDartboardLayoutCreate
    from ..models.user_dartboard_layout_update import UserDartboardLayoutUpdate
    from ..models.user_update import UserUpdate
    from ..models.view_create import ViewCreate
    from ..models.view_update import ViewUpdate


T = TypeVar("T", bound="Operation")


@_attrs_define
class Operation:
    """
    Attributes:
        kind (OperationKind): * `create` - CREATE
            * `delete` - DELETE
            * `update` - UPDATE
            * `update_list_add` - UPDATE_LIST_ADD
            * `update_list_remove` - UPDATE_LIST_REMOVE
        model (OperationModelKind): * `comment` - COMMENT
            * `comment_reaction` - COMMENT_REACTION
            * `dartboard` - DARTBOARD
            * `doc` - DOC
            * `event` - EVENT
            * `folder` - FOLDER
            * `layout` - LAYOUT
            * `event_subscription` - EVENT_SUBSCRIPTION
            * `relationship` - RELATIONSHIP
            * `relationship_kind` - RELATIONSHIP_KIND
            * `space` - SPACE
            * `status` - STATUS
            * `tag` - TAG
            * `task` - TASK
            * `task_attachment` - TASK_ATTACHMENT
            * `task_doc_relationship` - TASK_DOC_RELATIONSHIP
            * `task_link` - TASK_LINK
            * `tenant` - TENANT
            * `user` - USER
            * `user_dartboard_layout` - USER_DARTBOARD_LAYOUT
            * `view` - VIEW
        data (Union['CommentCreate', 'CommentReactionCreate', 'CommentReactionUpdate', 'CommentUpdate',
            'DartboardCreate', 'DartboardUpdate', 'DocCreate', 'DocUpdate', 'EventCreate', 'EventSubscriptionCreate',
            'EventSubscriptionUpdate', 'EventUpdate', 'FolderCreate', 'FolderUpdate', 'LayoutCreate', 'LayoutUpdate',
            'RelationshipCreate', 'RelationshipKindCreate', 'RelationshipKindUpdate', 'RelationshipUpdate', 'SpaceCreate',
            'SpaceUpdate', 'StatusCreate', 'StatusUpdate', 'TagCreate', 'TagUpdate', 'TaskAttachmentCreate',
            'TaskAttachmentUpdate', 'TaskCreate', 'TaskDocRelationshipCreate', 'TaskDocRelationshipUpdate',
            'TaskLinkCreate', 'TaskLinkUpdate', 'TaskUpdate', 'TenantCreate', 'TenantUpdate', 'UserCreate',
            'UserDartboardLayoutCreate', 'UserDartboardLayoutUpdate', 'UserUpdate', 'ViewCreate', 'ViewUpdate']):
    """

    kind: OperationKind
    model: OperationModelKind
    data: Union[
        "CommentCreate",
        "CommentReactionCreate",
        "CommentReactionUpdate",
        "CommentUpdate",
        "DartboardCreate",
        "DartboardUpdate",
        "DocCreate",
        "DocUpdate",
        "EventCreate",
        "EventSubscriptionCreate",
        "EventSubscriptionUpdate",
        "EventUpdate",
        "FolderCreate",
        "FolderUpdate",
        "LayoutCreate",
        "LayoutUpdate",
        "RelationshipCreate",
        "RelationshipKindCreate",
        "RelationshipKindUpdate",
        "RelationshipUpdate",
        "SpaceCreate",
        "SpaceUpdate",
        "StatusCreate",
        "StatusUpdate",
        "TagCreate",
        "TagUpdate",
        "TaskAttachmentCreate",
        "TaskAttachmentUpdate",
        "TaskCreate",
        "TaskDocRelationshipCreate",
        "TaskDocRelationshipUpdate",
        "TaskLinkCreate",
        "TaskLinkUpdate",
        "TaskUpdate",
        "TenantCreate",
        "TenantUpdate",
        "UserCreate",
        "UserDartboardLayoutCreate",
        "UserDartboardLayoutUpdate",
        "UserUpdate",
        "ViewCreate",
        "ViewUpdate",
    ]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.comment_create import CommentCreate
        from ..models.comment_reaction_create import CommentReactionCreate
        from ..models.comment_reaction_update import CommentReactionUpdate
        from ..models.comment_update import CommentUpdate
        from ..models.dartboard_create import DartboardCreate
        from ..models.dartboard_update import DartboardUpdate
        from ..models.doc_create import DocCreate
        from ..models.doc_update import DocUpdate
        from ..models.event_create import EventCreate
        from ..models.event_subscription_create import EventSubscriptionCreate
        from ..models.event_subscription_update import EventSubscriptionUpdate
        from ..models.event_update import EventUpdate
        from ..models.folder_create import FolderCreate
        from ..models.folder_update import FolderUpdate
        from ..models.layout_create import LayoutCreate
        from ..models.layout_update import LayoutUpdate
        from ..models.relationship_create import RelationshipCreate
        from ..models.relationship_kind_create import RelationshipKindCreate
        from ..models.relationship_kind_update import RelationshipKindUpdate
        from ..models.relationship_update import RelationshipUpdate
        from ..models.space_create import SpaceCreate
        from ..models.space_update import SpaceUpdate
        from ..models.status_create import StatusCreate
        from ..models.status_update import StatusUpdate
        from ..models.tag_create import TagCreate
        from ..models.tag_update import TagUpdate
        from ..models.task_attachment_create import TaskAttachmentCreate
        from ..models.task_attachment_update import TaskAttachmentUpdate
        from ..models.task_create import TaskCreate
        from ..models.task_doc_relationship_create import TaskDocRelationshipCreate
        from ..models.task_doc_relationship_update import TaskDocRelationshipUpdate
        from ..models.task_link_create import TaskLinkCreate
        from ..models.task_link_update import TaskLinkUpdate
        from ..models.task_update import TaskUpdate
        from ..models.tenant_create import TenantCreate
        from ..models.tenant_update import TenantUpdate
        from ..models.user_create import UserCreate
        from ..models.user_dartboard_layout_create import UserDartboardLayoutCreate
        from ..models.user_dartboard_layout_update import UserDartboardLayoutUpdate
        from ..models.user_update import UserUpdate
        from ..models.view_create import ViewCreate

        kind = self.kind.value

        model = self.model.value

        data: Dict[str, Any]

        if isinstance(self.data, CommentCreate):
            data = self.data.to_dict()

        elif isinstance(self.data, CommentUpdate):
            data = self.data.to_dict()

        elif isinstance(self.data, CommentReactionCreate):
            data = self.data.to_dict()

        elif isinstance(self.data, CommentReactionUpdate):
            data = self.data.to_dict()

        elif isinstance(self.data, DartboardCreate):
            data = self.data.to_dict()

        elif isinstance(self.data, DartboardUpdate):
            data = self.data.to_dict()

        elif isinstance(self.data, DocCreate):
            data = self.data.to_dict()

        elif isinstance(self.data, DocUpdate):
            data = self.data.to_dict()

        elif isinstance(self.data, EventCreate):
            data = self.data.to_dict()

        elif isinstance(self.data, EventUpdate):
            data = self.data.to_dict()

        elif isinstance(self.data, FolderCreate):
            data = self.data.to_dict()

        elif isinstance(self.data, FolderUpdate):
            data = self.data.to_dict()

        elif isinstance(self.data, LayoutCreate):
            data = self.data.to_dict()

        elif isinstance(self.data, LayoutUpdate):
            data = self.data.to_dict()

        elif isinstance(self.data, EventSubscriptionCreate):
            data = self.data.to_dict()

        elif isinstance(self.data, EventSubscriptionUpdate):
            data = self.data.to_dict()

        elif isinstance(self.data, RelationshipCreate):
            data = self.data.to_dict()

        elif isinstance(self.data, RelationshipUpdate):
            data = self.data.to_dict()

        elif isinstance(self.data, RelationshipKindCreate):
            data = self.data.to_dict()

        elif isinstance(self.data, RelationshipKindUpdate):
            data = self.data.to_dict()

        elif isinstance(self.data, SpaceCreate):
            data = self.data.to_dict()

        elif isinstance(self.data, SpaceUpdate):
            data = self.data.to_dict()

        elif isinstance(self.data, StatusCreate):
            data = self.data.to_dict()

        elif isinstance(self.data, StatusUpdate):
            data = self.data.to_dict()

        elif isinstance(self.data, TagCreate):
            data = self.data.to_dict()

        elif isinstance(self.data, TagUpdate):
            data = self.data.to_dict()

        elif isinstance(self.data, TaskCreate):
            data = self.data.to_dict()

        elif isinstance(self.data, TaskUpdate):
            data = self.data.to_dict()

        elif isinstance(self.data, TaskAttachmentCreate):
            data = self.data.to_dict()

        elif isinstance(self.data, TaskAttachmentUpdate):
            data = self.data.to_dict()

        elif isinstance(self.data, TaskDocRelationshipCreate):
            data = self.data.to_dict()

        elif isinstance(self.data, TaskDocRelationshipUpdate):
            data = self.data.to_dict()

        elif isinstance(self.data, TaskLinkCreate):
            data = self.data.to_dict()

        elif isinstance(self.data, TaskLinkUpdate):
            data = self.data.to_dict()

        elif isinstance(self.data, TenantCreate):
            data = self.data.to_dict()

        elif isinstance(self.data, TenantUpdate):
            data = self.data.to_dict()

        elif isinstance(self.data, UserCreate):
            data = self.data.to_dict()

        elif isinstance(self.data, UserUpdate):
            data = self.data.to_dict()

        elif isinstance(self.data, UserDartboardLayoutCreate):
            data = self.data.to_dict()

        elif isinstance(self.data, UserDartboardLayoutUpdate):
            data = self.data.to_dict()

        elif isinstance(self.data, ViewCreate):
            data = self.data.to_dict()

        else:
            data = self.data.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "kind": kind,
                "model": model,
                "data": data,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.comment_create import CommentCreate
        from ..models.comment_reaction_create import CommentReactionCreate
        from ..models.comment_reaction_update import CommentReactionUpdate
        from ..models.comment_update import CommentUpdate
        from ..models.dartboard_create import DartboardCreate
        from ..models.dartboard_update import DartboardUpdate
        from ..models.doc_create import DocCreate
        from ..models.doc_update import DocUpdate
        from ..models.event_create import EventCreate
        from ..models.event_subscription_create import EventSubscriptionCreate
        from ..models.event_subscription_update import EventSubscriptionUpdate
        from ..models.event_update import EventUpdate
        from ..models.folder_create import FolderCreate
        from ..models.folder_update import FolderUpdate
        from ..models.layout_create import LayoutCreate
        from ..models.layout_update import LayoutUpdate
        from ..models.relationship_create import RelationshipCreate
        from ..models.relationship_kind_create import RelationshipKindCreate
        from ..models.relationship_kind_update import RelationshipKindUpdate
        from ..models.relationship_update import RelationshipUpdate
        from ..models.space_create import SpaceCreate
        from ..models.space_update import SpaceUpdate
        from ..models.status_create import StatusCreate
        from ..models.status_update import StatusUpdate
        from ..models.tag_create import TagCreate
        from ..models.tag_update import TagUpdate
        from ..models.task_attachment_create import TaskAttachmentCreate
        from ..models.task_attachment_update import TaskAttachmentUpdate
        from ..models.task_create import TaskCreate
        from ..models.task_doc_relationship_create import TaskDocRelationshipCreate
        from ..models.task_doc_relationship_update import TaskDocRelationshipUpdate
        from ..models.task_link_create import TaskLinkCreate
        from ..models.task_link_update import TaskLinkUpdate
        from ..models.task_update import TaskUpdate
        from ..models.tenant_create import TenantCreate
        from ..models.tenant_update import TenantUpdate
        from ..models.user_create import UserCreate
        from ..models.user_dartboard_layout_create import UserDartboardLayoutCreate
        from ..models.user_dartboard_layout_update import UserDartboardLayoutUpdate
        from ..models.user_update import UserUpdate
        from ..models.view_create import ViewCreate
        from ..models.view_update import ViewUpdate

        d = src_dict.copy()
        kind = OperationKind(d.pop("kind"))

        model = OperationModelKind(d.pop("model"))

        def _parse_data(
            data: object,
        ) -> Union[
            "CommentCreate",
            "CommentReactionCreate",
            "CommentReactionUpdate",
            "CommentUpdate",
            "DartboardCreate",
            "DartboardUpdate",
            "DocCreate",
            "DocUpdate",
            "EventCreate",
            "EventSubscriptionCreate",
            "EventSubscriptionUpdate",
            "EventUpdate",
            "FolderCreate",
            "FolderUpdate",
            "LayoutCreate",
            "LayoutUpdate",
            "RelationshipCreate",
            "RelationshipKindCreate",
            "RelationshipKindUpdate",
            "RelationshipUpdate",
            "SpaceCreate",
            "SpaceUpdate",
            "StatusCreate",
            "StatusUpdate",
            "TagCreate",
            "TagUpdate",
            "TaskAttachmentCreate",
            "TaskAttachmentUpdate",
            "TaskCreate",
            "TaskDocRelationshipCreate",
            "TaskDocRelationshipUpdate",
            "TaskLinkCreate",
            "TaskLinkUpdate",
            "TaskUpdate",
            "TenantCreate",
            "TenantUpdate",
            "UserCreate",
            "UserDartboardLayoutCreate",
            "UserDartboardLayoutUpdate",
            "UserUpdate",
            "ViewCreate",
            "ViewUpdate",
        ]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_0 = CommentCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_1 = CommentUpdate.from_dict(data)

                return componentsschemas_operation_model_data_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_2 = CommentReactionCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_3 = CommentReactionUpdate.from_dict(data)

                return componentsschemas_operation_model_data_type_3
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_4 = DartboardCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_5 = DartboardUpdate.from_dict(data)

                return componentsschemas_operation_model_data_type_5
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_6 = DocCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_6
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_7 = DocUpdate.from_dict(data)

                return componentsschemas_operation_model_data_type_7
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_8 = EventCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_8
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_9 = EventUpdate.from_dict(data)

                return componentsschemas_operation_model_data_type_9
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_10 = FolderCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_10
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_11 = FolderUpdate.from_dict(data)

                return componentsschemas_operation_model_data_type_11
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_12 = LayoutCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_12
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_13 = LayoutUpdate.from_dict(data)

                return componentsschemas_operation_model_data_type_13
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_14 = EventSubscriptionCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_14
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_15 = EventSubscriptionUpdate.from_dict(data)

                return componentsschemas_operation_model_data_type_15
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_16 = RelationshipCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_16
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_17 = RelationshipUpdate.from_dict(data)

                return componentsschemas_operation_model_data_type_17
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_18 = RelationshipKindCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_18
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_19 = RelationshipKindUpdate.from_dict(data)

                return componentsschemas_operation_model_data_type_19
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_20 = SpaceCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_20
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_21 = SpaceUpdate.from_dict(data)

                return componentsschemas_operation_model_data_type_21
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_22 = StatusCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_22
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_23 = StatusUpdate.from_dict(data)

                return componentsschemas_operation_model_data_type_23
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_24 = TagCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_24
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_25 = TagUpdate.from_dict(data)

                return componentsschemas_operation_model_data_type_25
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_26 = TaskCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_26
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_27 = TaskUpdate.from_dict(data)

                return componentsschemas_operation_model_data_type_27
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_28 = TaskAttachmentCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_28
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_29 = TaskAttachmentUpdate.from_dict(data)

                return componentsschemas_operation_model_data_type_29
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_30 = TaskDocRelationshipCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_30
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_31 = TaskDocRelationshipUpdate.from_dict(data)

                return componentsschemas_operation_model_data_type_31
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_32 = TaskLinkCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_32
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_33 = TaskLinkUpdate.from_dict(data)

                return componentsschemas_operation_model_data_type_33
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_34 = TenantCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_34
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_35 = TenantUpdate.from_dict(data)

                return componentsschemas_operation_model_data_type_35
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_36 = UserCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_36
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_37 = UserUpdate.from_dict(data)

                return componentsschemas_operation_model_data_type_37
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_38 = UserDartboardLayoutCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_38
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_39 = UserDartboardLayoutUpdate.from_dict(data)

                return componentsschemas_operation_model_data_type_39
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_operation_model_data_type_40 = ViewCreate.from_dict(data)

                return componentsschemas_operation_model_data_type_40
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemas_operation_model_data_type_41 = ViewUpdate.from_dict(data)

            return componentsschemas_operation_model_data_type_41

        data = _parse_data(d.pop("data"))

        operation = cls(
            kind=kind,
            model=model,
            data=data,
        )

        operation.additional_properties = d
        return operation

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
